      SUBROUTINE yfszz(MODE,XPAR,NPAR)
*     ********************************
!=======================================================================
!=======================================================================
!============================= YFSZZ ===================================
!=======================================================================
!======================= YFS EXPONENTIATION ============================
!========================= IN e+e- --> ZZ ==============================
!===================== AND DECAYS ZZ --> 4-fermions ====================
!=======================================================================
!=======================================================================
!========================== YFSZZ VERSION 1.02 =========================
!=======================================================================
!============================ OCTOBER 1999 =============================
!=======================================================================
!=======================================================================
!= HISTORY: ============================================================
!= 1. YFSWW2 VERSION 1.00, AUGUST 1990 =================================
!= 2. YFSWW2 VERSION 2.xx, AUGUST 1995 -- MAY 1996 =====================
!= 3. YFSZZ  VERSION 1.00, JUNE 1996 ===================================
!= 4. YFSZZ  VERSION 1.01, APRIL1998 ===================================
!=======================================================================
!=======================================================================
! AUTHORS:
!    S. JADACH,   INSTITUTE OF NUCLEAR PHYSICS, CRACOW, POLAND
!    W. PLACZEK,  UNIVERSITY OF TENNESSEE, KNOXVILLE, TENNESSEE
!    B.F.L. WARD, UNIVERSITY OF TENNESSEE, KNOXVILLE, TENNESSEE
!=======================================================================
!
! MODE =-1/0/1/2 defines       
!       initialization/generation/give-xsection/final-report  
! npar(1) =KeyCor=0 born without any bremss.  
!          KeyCor=1 initial state radiation (ISR) only 
! npar(2) =KeyDZ1   decay channel of 1st Z 
! npar(3) =KeyDZ2   decay channel of 2nd Z 
!         =1-v_e, 2-v_mu 3-v_tau, 4-e, 5-mu, 6-tau, 7-u, 8-d, 9-c, 10-s, 11-b 
! npar(4) =KeyAcc=0,1 Anomalous ZZV (V=gamma,Z) couplings OFF/ON  
! npar(5) =KeyWgt=0,1 unweighted/weighted events 
!----------------------------------------------------------------- 
! xpar( 1) =CMSEne  =  CMS energy 
! xpar( 2) =amaZ    =  mass of Z  
! xpar( 3) =GammZ   =  width of Z 
! xpar( 4) =amaW    =  mass of W
! xpar( 5) =GammW   =  width of W  
! xpar( 6) =sinW2   =  sin^2(theta_Weinberg) 
! xpar( 7) =alfZin  =  1/alpha_Z, where alpha_Z is alpha QED at M_Z scale
! xpar( 8) =vvmin   =  minimum v-variable (dimesionless) =epsilon 
! xpar( 9) =vvmax   =  maximum v-variable   
! xpar(10) =WTMAX   =  maximum weight for rejection (unweighted events)
! xpar(21) - xpar(52) = values of anomalous ZZV couplings
!   (convention of K. Hagiwara, R.D. Peccei, D. Zeppenfeld and K. Hikasa, 
!                  Nucl. Phys. B282 (1987) 253):
!         g4(1)   = DCMPLX(xpar(21),xpar(31))
!         g5(1)   = DCMPLX(xpar(22),xpar(32))
!         ---------
!         g4(2)   = DCMPLX(xpar(41),xpar(51))
!         g5(2)   = DCMPLX(xpar(42),xpar(52))
! where                                                         (SM value)
!
!   g4(1)  : g_4^g, anomalous coupling for ZZg vertex            (0.0,0.0)
!   g5(1)  : g_5^g, anomalous coupling for ZZg vertex            (0.0,0.0)
!-------
!   g4(2)  : g_4^Z, anomalous coupling for ZZZ vertex            (0.0,0.0)
!   g5(2)  : g_5^Z, anomalous coupling for ZZZ vertex            (0.0,0.0)
!
! Other inputs in subroutine FILEXP.
!
!********************** OUTPUT *******************************************   
! 4-momenta and photon multiplicity are in:
!      COMMON / MOMSET / Q1(4),Q2(4),PHSUM(4),PHOT(100,4),NPHOT
! where Q1, Q2 are 4-momenta of Z's, resp., PHSUM is a sum of
! photon 4-momenta, while PHOT is an array of foton 4-momenta
! for NPHOT photon multiplicity. 
! The 4-momenta of ZZ-decay products can be found in:
!      COMMON / MOMDEC / PF1(4),PF2(4),PF3(4),PF4(4),PHOT(100,4),NPHOT 
! where PF1, PF2 are 4-momenta of 1st Z decay products, and PF3, PF4 are
! ones of 2nd Z decay products (PHOT and NPHOT as above).
!
! For KeyWgt=1 weighted events are generated and the user should   
! use the weight WTMOD from the common block:               
!     COMMON / WGTALL / WTMOD,WTCRU1,WTCRU2,WTSET(100)
! WTMOD is the actual model weight depending on the input parameters.       
!
! The total cross section, its relative error, the number of generated 
! events are provided in a post-generation mode (MODE=1,2) through:
! xpar(10)=XSECPB = cross section in picobarns        
! xpar(11)=ERREL  = relative error (dimensionless)  
! xpar(12)=XSMC   = cross section in R-units   
! npar(10)=NEVGEN = number of generated events      
!----------------------------------------------------------------------
! Last update: 22.10.1999              by: W.P.   
!======================================================================
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON / INOUT  / NINP,NOUT
      DIMENSION  XPAR(*),NPAR(*)
!
      IF(MODE.EQ.-1) 
     $WRITE(NOUT,6900)
     $'  *****************************************************',
     $'  *  ###   ### ########  ######  ######### #########  *',
     $'  *  ###   ### ######## ######## ########  ########   *',
     $'  *   ### ###  ###      ####         ###       ###    *',
     $'  *     ###    ######     ####      ###       ###     *',
     $'  *     ###    ######       ####   ###       ###      *',
     $'  *     ###    ###      ########  ########  ########  *',
     $'  *     ###    ###       ######  ######### #########  *',
     $'  *****************************************************',
     $' '
 6900 FORMAT('1',10(/,10X,A))
      CALL EXPAND(MODE,XPAR,NPAR)
      END

      SUBROUTINE EXPAND(MODE,XPAR,NPAR)     
C     ****************************************     
C  !!!!! LOGBOOK OF CORRECTIONS starting Nov. 91 !!!!!
C  R1 and R2 in KINEKR undefined
C  BREMKF, VVDIST moved to yfs3mod.f
C  VVDIST replaced with VVRHO from YFSFIG
C  VESK8A replaced by VESK1W
C  FUNSKO replaced by RHOSKO (change of name)
C  GAUSS replaced by GAUSJD
C  CMSENE shiftet to XPAR !!! 3 first entries in XPAR changed
C  CHBIN2 replaced by CHBIN1
C  XPAR(10,12) in output are swapped (to bhlumi convention)
*  Initialization of GLIBK in the generator
*  Bug with parameter(pi=...) corrected
C=======================================================================
C=======================================================================
C=============================YFS3======================================
C=====================FERMION PAIR PRODUCTION===========================
C===============INITIAL AND FINAL STATE EXPONENTIATION==================
C=======================================================================
C=======================================================================
C=======================YFS VERSION 3.3.0===============================
C=======================================================================
C=========================FEBRUARY  1993================================
C=======================================================================
C AUTHORS: 
C    S. JADACH, JAGELLONIAN UNIVERSITY, CRACOW, POLAND      
C    B.F.L. WARD, UNIVERSITY OF TENNESSEE, KNOXVILLE, TENNESSEE        
C=======================================================================
C  Adapted for Z-pair production by Wieslaw Placzek,   June 1996
C  Last update: 21.11.1996         by: W.P.
C=======================================================================
C          
C SOME CONTROLL HISTOGRAMING STILL IN PROGRAM      
C          
C generator of Ynnie-Fautschi-Suura type         
C with exponentiated single bremsstrahlung         
C********* input    
C mode =-1/0/1/2 defines       
C       initialization/generation/give-xsection/final-report  
C cmsene   = centre of mass energy (gev)  
C npar(1)=keyrad=1000001 initial state only        
C         keyrad=1000010 final state only          
C         keyrad=1000011 initial + final state     
C         keyrad=1000000 born without any bremss.  
C         keyrad=100n001 fixed initial state multiplicity (tests)      
C         keyrad=10k0010 fixed final   state multiplicity (tests)      
C npar(2)=keyred=0,1,2   three different reduction procedures          
C npar(3)=keywgt=0,1   unweighted/weighted events  
C xpar(1)=cmsene  =  mass of z0  
C xpar(2)=amaz    =  mass of z0  
C xpar(3)=sinw2   =  sin(thetaweinberg)**2  
C xpar(4)=gammz   =  width of z0 
C xpar(5)=amfin   =  mass of final fermion  
C xpar(6)=vvmin   =  minimum v-variable (dimesionless) =epsilon 
C xpar(7)=vvmax   =  maximum v-variable   
C********* output   
C Four-momenta of and photon multiplicity in /momset/  
C xpar(10)=xsecnb = cross section in nanobarns        
C xpar(11)=errel  = relative error (dimensionless)  
C xpar(12)=xsmc   = cross section in r-units   
C npar(10)=nevacc = number of generated events      
C**************************    
C For advanced users only:     
C (1) One may use KEYRAD < 0  for running at fixed v-variable,         
C in this case all events have  v=VMAX precisely.  
C (2) For KEYWGT=1 weighted events are generated and the user should   
C use the weight WTMOD from the common block /WGTALL/.      
C WTMOD is the actual model weight depending on other input params.    
C The other interesting possibility is to use      
C     WT=WTCRU1*WTCRU2*WTSET(i) where   
C     WTSET(71) =   zero-th order initial+final    
C     WTSET(72) =   first order   initial+final    
C     WTSET(73) =   second order  initial+final    
C and the following provide the corresponding components of x-section. 
C     WTSET(80) =   First order, beta0 contribution alone   
C     WTSET(81) =   First order, beta1 contribution alone   
C     WTSET(90) =   Second order, beta0 contribution alone  
C     WTSET(91) =   Second order, beta1 contribution alone  
C     WTSET(92) =   Second order, beta2 contribution alone  
C furthermore, for the initial state alone we provide       
C     WTSET( 1) =   zero-th order initial          
C     WTSET( 2) =   first order   initial          
C     WTSET( 3) =   second order  initial          
C and the corresponding components      
C     WTSET(20) =   First order, beta0 contribution alone   
C     WTSET(21) =   First order, beta1 contribution alone   
C     WTSET(30) =   Second order, beta0 contribution alone  
C     WTSET(31) =   Second order, beta1 contribution alone  
C     WTSET(32) =   Second order, beta2 contribution alone  
C N.B. WTMOD=WTCRU1*WTCRU2*WTSET(71)    
C     ************************ 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)          
      PARAMETER(PI=3.1415926535897932D0,ALFINV=137.03604D0) 
      DIMENSION  XPAR( *),NPAR( *)      
      COMMON / CMONIT/ AVERWT,ERRELA,NEVTOT,NEVACC,NEVNEG,NEVOVE,NEVZER
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT      
      COMMON / MOMINI / XF1(4),XF2(4),XPHUM(4),XPHOT(100,4),NPHOX      
      COMMON / MOMFIN / YF1(4),YF2(4),YPHUM(4),YPHOT(100,4),NPHOY      
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF 
      COMMON / Wparam / amaW,GammW
      COMMON / WGTALL / WTMOD,WTCRU1,WTCRU2,WTSET(100)      
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT
      COMMON / KeyOth / KeyCor,KeyDWm,KeyDWp,KeyAcc
      COMMON / INOUT  / NINP,NOUT   
      COMMON / BXFMTS / BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G 
      CHARACTER*80      BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G 
      DIMENSION XXF(4)
      SAVE   / CMONIT /,/ MOMSET /,/ MOMINI /,/ MOMFIN /,/ WEKING /
      SAVE   / WGTALL /,/ KEYYFS /,/ INOUT  /,/ BXFMTS /
      SAVE   / Wparam /,/ KeyOth /
      SAVE   NEVGEN,KEYBIN,KEYBFI,CMSENE
      SAVE   IDYFS, WTMAX
      COMMON / VVREC  / VVMIN,VVMAX,VV,BETI
      SAVE   / VVREC  /
      COMMON / KOPLUT / V(4,5),A(4,5),B(5,5,5),XM2(5),XMG(5),CLR(4,5,2)
      SAVE   / KOPLUT /
      DOUBLE PRECISION R(100)
!
      IF(MODE.EQ.-1) THEN      
C     ==================================================================
C     =====================INITIALIZATION===============================
C     ==================================================================

! 6900 FORMAT('1',10(/,10X,A)) 
!      WRITE(NOUT,6900)  
!     $'  *************************************************************',
!     $'  *  ****   ****  **********    *******          ***********  *',
!     $'  *  ****   ****  **********  **********         **********   *',
!     $'  *   **** ****   ****        *****                 ******    *',
!     $'  *    ******     ********      ******    *****    ******     *',
!     $'  *     ****      ********         *****  *****      ******   *',
!     $'  *     ****      ****        **********         ***********  *',
!     $'  *     ****      ****         *******           **********   *',
!     $'  *************************************************************',
!     $' '   

* ==========================================================
* Initialization of internal histogramming package GLIBK
* Let us keep for YFS3 the GLIBK ID-ent range from 2 to 1000 
* ==========================================================
!WP: moved to the MAIN program block
!WP      CALL GLIMIT(20000)
!WP      CALL GOUTPU(NOUT)  

      CALL FILEXP(XPAR,NPAR)
!WP   
      CALL RANDA(8,R,-1)  

      IDYFS = 0
C GMONIT MONITORS WEIGHTS       
      CALL GMONIT(-1,IDYFS+71,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+72,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+73,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+74,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+75,0D0,1D0,1D0) 
      CALL GMONIT(-1,IDYFS+76,0D0,1D0,1D0) 
      CALL GMONIT(-1,IDYFS+77,0D0,1D0,1D0)    
      CALL GMONIT(-1,IDYFS+78,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+79,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+80,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+81,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+90,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+91,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+92,0D0,1D0,1D0)  
      CMSENE = 2D0*ENE
      WTMAX  = XPAR(10)
      KEYBIN = MOD(KEYBRM,10)   
      KEYBFI = MOD(KEYBRM,100)/10       
      IF(KEYBIN.EQ.1) CALL KARLUD(-1,DUM1,DUM2)     
      IF(KEYBFI.EQ.1) CALL KARFIN(-1, XXF,DUMM1,DUMM2,DUMM3,DUMM4)      
      CALL  MODEL(-1,DUM1,DUM2) 
      NEVGEN=0 
     
      ELSEIF(MODE.EQ.0) THEN    
C     ==================================================================
C     =====================GENERATION===================================
C     ==================================================================
      NEVGEN=NEVGEN+1   
  100 CONTINUE      
C ===================   
C ...Initial state YFS2 type generator  
         IF(KEYBIN.EQ.1) THEN   
            CALL KARLUD( 0,WTKARL,DUM2)     
         ELSE       
!WP            CALL GIBEA(CMSENE,AMFIN,XF1,XF2)        
            CALL KINEKR
            WTKARL = 1D0        
            NPHOX  = 0  
            VV = 0
         ENDIF      
C ...Four-momentum of virtual Z 
         DO 27 K=1,4    
  27     XXF(K) = XF1(K)+XF2(K)  
C ===================   
C ...Final state momenta in the Z frame     
         IF(KEYBFI.EQ.1 .AND. WTKARL.NE.0D0) THEN   
!WP            CALL KARFIN(0, XXF,AMFIN,WTCFIN)
            CALL KARFIN(0, XXF,AMFIN,AMFIN,AMFIN,WTCFIN)
         ELSE       
C ...No final state bremss, fermion momenta defined in Z frame  
!WP            CALL KINF2( XXF,AMFIN,YF1,YF2)  
            DO K = 1,4
              YF1(K) = XF1(K)
              YF2(K) = XF2(K)
            ENDDO
!
            WTCFIN = 1D0        
            NPHOY  = 0  
         ENDIF      
C ...Merging momenta and transform to CMS frame     
         CALL MERGIF 
C =================== All kinematics is fixed at this point =======     
         WTCRU1 = WTKARL *(1d0 - VV) 
         WTCRU2 = WTCFIN       
         WTCRUD = WTCRU1*WTCRU2  
C =================== All weights are fixed at this point =========     
CCCC     IF(NEVGEN.LE.5) CALL DUMPI(NOUT)   
CCCC     IF(NEVGEN.LE.5) CALL DUMPF(NOUT)   
C ...Model weight  
         CALL MODEL(0,WTMXX,WTMDL)
C ...principal model weight for WT=1 events (after rejection)   
         WT     = WTCRUD*WTMDL  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!WP: Kinematics of ZZ decays
         CALL KinDec 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C ...Weigt monitoring   
         CALL GMONIT(0,IDYFS+73,WTCRUD*WTSET(73),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+72,WTCRUD*WTSET(72),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+71,WTCRUD*WTSET(71),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+75,WTCRUD*(WTSET(73)-WTSET(72)),WTMAX,0D0)
         CALL GMONIT(0,IDYFS+74,WTCRUD*(WTSET(72)-WTSET(71)),WTMAX,0D0)
         CALL GMONIT(0,IDYFS+80,WTCRUD*WTSET(80),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+81,WTCRUD*WTSET(81),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+90,WTCRUD*WTSET(90),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+91,WTCRUD*WTSET(91),WTMAX,0D0)       
         CALL GMONIT(0,IDYFS+92,WTCRUD*WTSET(92),WTMAX,0D0)
!WP: Monitoring WT>WTMAX and WT<0 
         WTOVR = MAX(0D0,WT-WTMAX) 
         CALL GMONIT(0,IDYFS+76,WTOVR,0D0,0D0)
         WTNEG = MIN(WT,0D0)
         CALL GMONIT(0,IDYFS+77,WTNEG,0D0,0D0)  
C ...Rejection according to principal weight  
         IF(KEYWGT.EQ.0) THEN    
C ...unweihgted events with WT=1 
            CALL VARRAN(RN,1)
            CALL GMONIT(0,IDYFS+79,WT ,WTMAX,RN)
            IF(RN.GT.WT/WTMAX) GOTO 100       
            WTMOD=1.D0    
C ...WTCRU1,2  weights are RESET to one 
            WTCRU1=1D0    
            WTCRU2=1D0    
         ELSE       
C ...weighted events      
            WTMOD  = WT 
            CALL GMONIT(0,IDYFS+79,WT     ,WTMAX,0D0)      
            CALL GMONIT(0,IDYFS+78,WTCRUD ,WTMAX,0D0)      
         ENDIF 
      ELSEIF(MODE.EQ.1.OR.MODE.EQ.2) THEN     
C     ==================================================================
C     =====================FINAL WEIGHT ANALYSIS========================
C     ==================================================================
      NPAR(10)= NEVGEN  
      NPAR(20)= NEVGEN   
      GNANOB=389.385D-30*1.D33   
      SIG0FB =  4D0*PI/(ALFINV**2*3D0*CMSENE**2)*GNANOB *1d6     
      XBORFB =  BORNY(CMSENE**2)        
      IF(KEYBIN.EQ.1) THEN       
         CALL KARLUD(MODE,XKARL,ERKARL)       
      ELSE   
         XKARL  = BORNY(CMSENE**2)      
         ERKARL = 0D0     
      ENDIF  
      IF(KEYBFI.EQ.1) CALL KARFIN(MODE,XXF,DUMM1,DUMM2,DUMM3,DUMM4)     
C     
      CALL GMONIT(1,IDYFS+79,AWTOT,DWTOT,DUMM3)     
      NPAR(10)= NEVGEN  
!WP      XSMC   =  XKARL*AVERWT     
!WP      EREL   =  SQRT(ERKARL**2+ERRELA**2)     
!WP      ERABS  =  XSMC*EREL        
!WP      XSMCPB =  XSMC*SIG0PB      
!WP      ERABS2 =  XSMCPB*EREL  
!WP: Different normalization of WW Born x-section 
      XSMCFB =  XKARL*AVERWT
      EREL   =  SQRT(ERKARL**2+ERRELA**2) 
      ERABS2 =  XSMCFB*EREL 
      XSMC   =  XSMCFB/SIG0FB 
      ERABS  =  XSMC*EREL 
      NEVT = NEVTOT
      NEVA = NEVGEN
      NEVN = NEVNEG
      NEVO = NEVOVE
!        
      XPAR(10) = XSMCFB
      XPAR(11) = EREL      
      XPAR(12) = XSMC  
!WP: x-sections for WT>WTMAX and WT<0 
      CALL GMONIT(1,IDYFS+76,AWT76,DWT76,DUMM3)
      xsove = AWT76/AWTOT
      dsove = xsove*DWT76
      CALL GMONIT(1,IDYFS+77,AWT77,DWT77,DUMM3)
      xsneg = AWT77/AWTOT
      dsneg = xsneg*DWT77
      IF(KEYWGT.EQ.0) THEN              
C ...Unweighted events, normal option     
         XPAR(20)=XSMCFB            
         XPAR(21)=EREL                  
         XPAR(22)=XSMC
      ELSE          
C ...Weighted events, additional information on x-sections  
         CALL GMONIT(1,IDYFS+78,DUMM1,DUMM2,DUMM3)                
         XPAR(20)= XKARL    
         XPAR(21)= ERRELA               
         XPAR(22)= XKARL /SIG0FB  
      ENDIF         
C NO PRINTOUT FOR MODE =1               
      IF(MODE.EQ.1) RETURN              
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) 'EXPAND output - window A'              
      WRITE(NOUT,BXL1F) CMSENE,       'CMS energy total  ','CMSENE','A0'
      WRITE(NOUT,BXL2F) XSMCFB,ERABS2,'xs_tot [fb]       ','XSMCFB','A1'
!WP   WRITE(NOUT,BXL1F) XSMC,         'xs_tot [R-units]  ','XSMC  ','A3'
!WP   WRITE(NOUT,BXL1F) ERABS,        'absolute error [R]','ERABS ','A4'
      WRITE(NOUT,BXL1F) EREL,         'relative error    ','EREL  ','A5'
      WRITE(NOUT,BXL1I) NEVT,         'total no of events','NEVTOT','A6'
      WRITE(NOUT,BXL1I) NEVA,         'accepted    events','NEVACC','A7'
      WRITE(NOUT,BXL1I) NEVN,         'WT<0        events','NEVNEG','A8'
      WRITE(NOUT,BXL2F) xsneg,dsneg,  'xsec/xtot: WT<0   ','XSNEG ','A9'
      WRITE(NOUT,BXL1I) NEVO,        'WT>WTMAX    events','NEVOVE','A10'
      WRITE(NOUT,BXL2F) xsove,dsove, 'xsec/xtot:WT>WTMAX','XSOVE ','A11'
      WRITE(NOUT,BXL1F) WTMAX ,      'WTMAX             ','WTMAX ','A12'
!WP   WRITE(NOUT,BXL1F) XBORPB,      'xs_Born   picob.  ','XBORPB','A13'
      WRITE(NOUT,BXCLO)         
C     ==================================================================
C     =============SUPPL. FINAL WEIGHT ANALYSIS=========================
C     ==================================================================
      WRITE(NOUT,BXOPE)                 
      WRITE(NOUT,BXTXT) '        EXPAND output - window B ' 
      WRITE(NOUT,BXTXT) '           X-sections in fb   ' 
      CALL GMONIT(1,IDYFS+73,DUMM1,DUMM2,DUMM3)                   
      XS03   =  XKARL*AVERWT            
      DXS03  =  XKARL*AVERWT*SQRT(ERKARL**2+ERRELA**2)      
      CALL GMONIT(1,IDYFS+72,DUMM1,DUMM2,DUMM3)                   
      XS02   =  XKARL*AVERWT            
      DXS02  =  XKARL*AVERWT*SQRT(ERKARL**2+ERRELA**2)      
      CALL GMONIT(1,IDYFS+71,DUMM1,DUMM2,DUMM3)                   
      XS01   =  XKARL*AVERWT            
      DXS01  =  XKARL*AVERWT*SQRT(ERKARL**2+ERRELA**2)      
      CALL GMONIT(1,IDYFS+75,DUMM1,DUMM2,DUMM3)                   
      XS05   =  XKARL*AVERWT            
      DXS05  =  XKARL*AVERWT*SQRT(ERKARL**2+ERRELA**2)      
      CALL GMONIT(1,IDYFS+74,DUMM1,DUMM2,DUMM3)                   
      XS04   =  XKARL*AVERWT            
      DXS04  =  XKARL*AVERWT*SQRT(ERKARL**2+ERRELA**2)      
      WRITE(NOUT,BXL2F) XS03,DXS03,'X-section','O(alf2)',  'B1'
      WRITE(NOUT,BXL2F) XS02,DXS02,'X-section','O(alf1)',  'B2'
      WRITE(NOUT,BXL2F) XS01,DXS01,'X-section','O(alf0)',  'B3'
      IF(XS02.NE.0D0) WRITE(NOUT,BXL2F)                     
     $ XS05/XS02,DXS05/XS02,'(O(alf2)-O(alf1))','/O(alf1)','B4'
      IF(XS01.NE.0D0) WRITE(NOUT,BXL2F)                     
     $ XS04/XS01,DXS04/XS01,'(O(alf1)-O(alf0))','/O(alf0)','B5'
      WRITE(NOUT,BXCLO)                 
C -------------     
      WTCRU1 = XKARL                    
      WTCRU2 = ERKARL                   
      CALL  MODEL(MODE,DUMM1,DUMM2)     
C -------------     
      ELSE          
C     ====          
      WRITE(NOUT,*) '===>EXPAND: WRONG MODE'                
      STOP          
      ENDIF         
C     =====         
      END
                 
      SUBROUTINE MERGIF   
C     *****************   
C Merging two commons all photons together  
C Transformation to common (CMS) trame is supposed to be already done  
C     ************************************  
      IMPLICIT DOUBLE PRECISION  (A-H,O-Z)  
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT  
      COMMON / MOMINI / XF1(4),XF2(4),XPHUM(4),XPHOT(100,4),NPHOX 
      COMMON / MOMFIN / YF1(4),YF2(4),YPHUM(4),YPHOT(100,4),NPHOY 
C ...Photons              
      NPHOT  = 0          
      DO 130 I=1,NPHOX    
        NPHOT  =NPHOT+1   
        DO 130 K=1,4      
  130   SPHOT(NPHOT,K)=XPHOT(I,K)           
      DO 140 I=1,NPHOY    
        NPHOT  =NPHOT+1   
        DO 140 K=1,4      
  140   SPHOT(NPHOT,K)=YPHOT(I,K)           
C ...Final state fermions                   
      DO 145 K=1,4        
        QF1(K)= YF1(K)    
  145   QF2(K)= YF2(K)    
C ...Axiliary             
      DO 150 K=1,4        
  150    SPHUM(K)= XPHUM(K)+YPHUM(K)        
      END 
                
      SUBROUTINE FILEXP(XPAR,NPAR)   
C     ***********************************   
C TRANSFERS AND DEFINES INPUT PARAMS, PRINTS INPUT PARAMETERS 
! Modified by Wieslaw Placzek,       Knoxville, June 1996
! Last update: 16.07.1996       by: WP
      IMPLICIT DOUBLE PRECISION  (A-H,O-Z)  
      DIMENSION  XPAR( *),NPAR( *)          
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF 
      COMMON / Wparam / amaW,GammW
      COMMON / DECDAT / AMAFIN(20), BR(20), ICOD(20)
      COMMON / KeyOth / KeyCor,KeyDZ1,KeyDZ2,KeyAcc
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT    
      COMMON / SPISET / POLAR1,POLAR2,POLFI1,POLFI2           
      COMMON / UUREC  / UU,EPS,DELTA        
      COMMON / VVREC  / VVMIN,VVMAX,VV,BETI                   
      COMMON / INOUT  / NINP,NOUT 
      COMMON /KOPLIN/ ALFA,SW2,ALFAS,XMZ,XMW,XMT
      COMMON /INFORP/CCW,WCC,PHHI,R(8)
      COMMON / BXFMTS / BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G 
      CHARACTER*80      BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G 
      CHARACTER*80      BXL2C            
      COMMON / RANPAR / KEYRND
!   --Anomalous WWV Couplings
      COMMON / AnccZZ / g4(2),g5(2)
      COMPLEX*16 g4,g5
      SAVE / WEKING /,/ KEYYFS /,/ SPISET /,/ UUREC  /,/ VVREC  /
      SAVE / INOUT  /,/ BXFMTS /,/ RANPAR /
      SAVE / Wparam /,/ DECDAT /,/ KeyOth /,/ AnccZZ /, /KOPLIN/
!
! ...BX-formats for nice and flexible outputs                 
      BXOPE =  '(//1X,15(5H*****)    )'     
      BXTXT =  '(1X,1H*,                  A48,25X,    1H*)'   
      BXL1I =  '(1X,1H*,I17,                 16X, A20,A12,A7, 1X,1H*)'
      BXL1F =  '(1X,1H*,F17.8,               16X, A20,A12,A7, 1X,1H*)' 
      BXL2F =  '(1X,1H*,F17.8, 4H  +-, F11.8, 1X, A20,A12,A7, 1X,1H*)'
      BXL1G =  '(1X,1H*,G17.8,               16X, A20,A12,A7, 1X,1H*)'
      BXL2G =  '(1X,1H*,G17.8, 4H  +-, F11.8, 1X, A20,A12,A7, 1X,1H*)'
      BXCLO =  '(1X,15(5H*****)/   )'  
      BXL2C ='(1X,1H*,1H(,F14.8,3H +i,F13.7,1H),1X,A20,A12,A7,1X,1H*)' 

!  type of the RANDOM NUMBER GENERATOR RANMAR                   
      KEYRND = 1
!----------   
      KeyCor = npar( 1)
      KeyDZ1 = npar( 2)
      KeyDZ2 = npar( 3)
      KeyAcc = npar( 4)     
      KEYWGT = npar( 5)
! Set-up original YFS3 switches     
      IF (KeyCor.EQ.0) THEN
        KEYBRM = 1000000  
      ELSE
        KEYBRM = 1000001
      ENDIF
      KEYFIX = 0           
      KEYRED = 1 
      KEYZET = 2 
!-------------------------------    
      CMSEne = xpar( 1)      
      amaZ   = xpar( 2)      
      GammZ  = xpar( 3)      
      amaW   = xpar( 4) 
      GammW  = xpar( 5)   
      sinW2  = xpar( 6)
      alfZin = xpar( 7)
      vvmin  = xpar( 8)      
      vvmax  = xpar( 9)
      WTMAX  = xpar(10)
      IF (WTMAX.LT.0d0) THEN
        WTMAX = 7.0
        xpar(10) = WTMAX
      ENDIF
      ENE    = CMSENE/2D0      
      EPS    = VVMIN       
      DELTA  = EPS*1D-3    
      POLAR1 = 0D0          
      POLAR2 = 0D0          
      POLFI1 = 0D0          
      POLFI2 = 0D0          
      AMEL   = 0.51099906D-3
      AMFIN  = amaZ
      IDE=2               
      IDF=2               
      XK0=3.D-3           
!
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) '               YFSZZ version 1.02,  Oct. 1999'
      WRITE(NOUT,BXTXT) '                     INPUT PARAMATERS:       '
      WRITE(NOUT,BXL1I) KeyCor,     'General QED switch ','KeyCor','N1'
      WRITE(NOUT,BXL1I) KeyDZ1,     'Decay channel of Z1','KeyDZ1','N2'
      WRITE(NOUT,BXL1I) KeyDZ2,     'Decay channel of Z2','KeyDZ2','N3'
      WRITE(NOUT,BXL1I) KeyAcc,     'Anom. Coup. switch ','KeyAcc','N4'
      WRITE(NOUT,BXL1I) KeyWgt,     'Weighting  switch  ','KeyWgt','N5'
      WRITE(NOUT,BXL1F) CMSEne,     'CMS energy total   ','CMSEne','X1'
      WRITE(NOUT,BXL1F) amaZ,       'Z mass   [GeV]     ','amaZ  ','X2'
      WRITE(NOUT,BXL1F) GammZ,      'Z width  [GeV]     ','GammZ ','X3'
      WRITE(NOUT,BXL1F) amaW,       'W mass   [GeV]     ','amaZ  ','X4'
      WRITE(NOUT,BXL1F) GammW,      'W width  [GeV]     ','GammZ ','X5'
      WRITE(NOUT,BXL1F) sinW2,      'sin^2(theta_W)     ','sinW2 ','X6'
      WRITE(NOUT,BXL1F) alfZin,     '1/alpha_Z          ','alfZin','X7'
      WRITE(NOUT,BXL1F) vvmin,      'dummy infrared cut ','vvmin ','X8'
      WRITE(NOUT,BXL1F) vvmax,      'v_max ( =1 )       ','vvmax ','X9'
      WRITE(NOUT,BXL1F) WTMAX,      'Maximum weight     ','WTMAX ','X10'
      WRITE(NOUT,BXCLO)
!
! Additional Input parameters:
! Fermion masses (including t-quark mass)
      AMAFIN(1)= 10D-3          ! d -1/3
      AMAFIN(2)=  5D-3          ! u +2/3
      AMAFIN(3)= 0.2D0          ! s -1/3
      AMAFIN(4)= 1.3D0          ! c +2/3
      AMAFIN(5)= 4.3D0          ! b -1/3
      AMAFIN(6)= 174.0D0        ! t +2/3
!         
      AMAFIN(11)= 0.51099906D-3 ! e
      AMAFIN(12)= 0D0           ! v_e
      AMAFIN(13)= 0.105658389D0 ! mu
      AMAFIN(14)= 0D0           ! v_mu
      AMAFIN(15)= 1.7771D0      ! tau
      AMAFIN(16)= 0D0           ! v_tau
      WRITE(NOUT,BXOPE)     
      WRITE(NOUT,BXTXT) '                     Fermion masses:         '
      WRITE(NOUT,BXL1F) AMAFIN(1),     ' d','AMAFIN( 1)','IM1'
      WRITE(NOUT,BXL1F) AMAFIN(2),     ' u','AMAFIN( 2)','IM2'
      WRITE(NOUT,BXL1F) AMAFIN(3),     ' s','AMAFIN( 3)','IM3'
      WRITE(NOUT,BXL1F) AMAFIN(4),     ' c','AMAFIN( 4)','IM4'
      WRITE(NOUT,BXL1F) AMAFIN(5),     ' b','AMAFIN( 5)','IM5'
      WRITE(NOUT,BXL1F) AMAFIN(6),     ' t','AMAFIN( 6)','IM6'
      WRITE(NOUT,BXL1F) AMAFIN(11),    ' e','AMAFIN(11)','IM7'
      WRITE(NOUT,BXL1F) AMAFIN(12),    've','AMAFIN(12)','IM8'
      WRITE(NOUT,BXL1F) AMAFIN(13),    'mu','AMAFIN(13)','IM9'
      WRITE(NOUT,BXL1F) AMAFIN(14),   'vmu','AMAFIN(14)','IM10'
      WRITE(NOUT,BXL1F) AMAFIN(15),   'tau','AMAFIN(15)','IM11'
      WRITE(NOUT,BXL1F) AMAFIN(16),  'vtau','AMAFIN(16)','IM12'
      WRITE(NOUT,BXCLO)
!       
!----------------------------------------------------------------------!
! Setting up the anomalous couplings as given in the paper:            !
!     K. Hagiwara, R.D. Peccei, D. Zeppenfeld and K. Hikasa,           !
!                 Nucl. Phys. B282 (1987) 253.                         !
! The variables used in this routine correspond to the following       !
! contants defined in the above paper:                                 !
!           constant name     corresponding variable                   ! 
!                g_4^V                g4(2)                            !
!                g_5^V                g5(2)                            !
!----------------------------------------------------------------------!      
       IF(KeyAcc .EQ.0) THEN
!     --Set up Standard Model constants:
!     --for ZZgamma vertex
        g4(1)   = (0.0,0.0)
        g5(1)   = (0.0,0.0)
!     --WWZ vertex
        g4(2)   = (0.0,0.0)
        g5(2)   = (0.0,0.0)
      ELSEIF(KeyAcc.EQ.1) THEN
!     --Set up constants OTHER than SM (provided by the user):
!     --for ZZgamma vertex
        g4(1)   = DCMPLX(xpar(21),xpar(31))
        g5(1)   = DCMPLX(xpar(22),xpar(32))
!     --WWZ vertex
        g4(2)   = DCMPLX(xpar(41),xpar(51))
        g5(2)   = DCMPLX(xpar(42),xpar(52))
!
        WRITE(NOUT,BXOPE)     
        WRITE(NOUT,BXTXT)'Anomalous ZZV Couplings Activated'
        WRITE(NOUT,BXTXT)'Convention from:'
        WRITE(NOUT,BXTXT)
     $         'K.Hagiwara, R.D.Peccei, D.Zeppenfeld, K.Hikasa,'
        WRITE(NOUT,BXTXT)'                Nucl. Phys. B282 (1987) 253.'
        WRITE(NOUT,BXTXT)'                    for ZZgamma vertex'  
        WRITE(NOUT,BXL2C) g4(1),             'g_4^g','g4(1)  ','IC21'
        WRITE(NOUT,BXL2C) g5(1),             'g_5^g','g5(1)  ','IC22'     
        WRITE(NOUT,BXTXT)'                        for ZZZ vertex'  
        WRITE(NOUT,BXL2C) g4(2),             'g_4^Z','g4(2)  ','IC41'
        WRITE(NOUT,BXL2C) g5(2),             'g_5^Z','g5(2)  ','IC42'     
        WRITE(NOUT,BXCLO)       
      ELSE
        write(6,*)'FILEXP==> Wrong KeyAcc: ',KeyAcc
        STOP
      ENDIF
! Initializations in ZZ routines
! ... strong coupling constant alpha_s (should be =0 in this version!)
      ALFAS = 0
!WP      ALFAS =0.123
      XMZ   = amaZ
      XMW   = amaW
      SW2   = sinW2
      ALFAI = alfZin 
      ALFA  = 1D0/ALFAI
      XMT   = AMAFIN(6)
      WCC   = 1d0
      SVAR  = CMSEne**2
      CALL KOPPEL
      CALL BORNV(SVAR,0d0)
      END 
                
      SUBROUTINE KARLUD(MODE,PAR1,PAR2)     
C     *********************************     
C LOW LEVEL  MONTE-CARLO GENERATOR          
C ADMINISTRATES DIRECTLY GENERATION OF V-VARIABLE             
C AND INDIRECTLY OF ALL OTHER VARIABLES.    
C     ************************              
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)   
      PARAMETER (PI=3.1415926535897932D0,ALFINV= 137.03604D0)      
      PARAMETER (CEULER =0.57721566D0)                  
      PARAMETER (NMAX= 40)   
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF 
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT    
C COMMUNICATES WITH VESKO/RHOSKO            
      COMMON / VVREC  / VVMIN,VVMAX,VV,BETI                   
      COMMON / INOUT  / NINP,NOUT     
C COMMUNICATES WITH GMONIT                  
      COMMON / CMONIT/ AVERWT,ERRELA,NEVTOT,NEVACC,NEVNEG,NEVOVE,NEVZER 
      COMMON / BXFMTS / BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G
      CHARACTER*80      BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G
!WP: weight (WCC) from Born-like WW process
      COMMON /INFORP/CCW,WCC,PHI,R(8)

      SAVE / WEKING /,/ KEYYFS /,/ VVREC  /,/ INOUT  /
      SAVE / CMONIT /,/ BXFMTS /,/INFORP/
      SAVE SVAR,BETI2,GAMFAP,GAMFAC,GAMFA2,PREC,XCGAUS,XDEL,XCVESK
      SAVE IDYFS
      EXTERNAL RHOSKO
C MAXIMUM PHOTON MULTIPLICITY               
C       
      IF(MODE.EQ.-1) THEN                   
C     ==================================================================
C     ===================INITIALIZATION=================================
C     ==================================================================
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) 'Initialize KARLUD  start'              
      SVAR = 4D0*ENE**2
      BETI = 2D0/ALFINV/PI*(DLOG(4D0*ENE**2/AMEL**2)-1D0)     
      BETI2= 2D0/ALFINV/PI* DLOG(4D0*ENE**2/AMEL**2)          
      GAMFAP =1D0-PI**2*BETI**2/12D0        
      GAMFAC =EXP(-CEULER*BETI)/DPGAMM(1D0+BETI)               
      GAMFA2 =EXP(-CEULER*BETI2)/DPGAMM(1D0+BETI2)             
      IDYFS = 0
      CALL GBOOK1(IDYFS+ 7,'WT IN KARLUD         $', 50 , 0D0,5D0)
      CALL GMONIT(-1,IDYFS+51,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+52,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+53,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+54,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+55,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+56,0D0,1D0,1D0)  
      CALL GMONIT(-1,IDYFS+59,0D0,1D0,1D0)  
      CALL VESK1W(-1,RHOSKO,XCVESK,DUM1,DUM2)             
      PREC = 1D-6
      XCGAUS =BREMKF(1,PREC)   
      XDEL = XCVESK/XCGAUS-1
      WRITE(NOUT,BXL1F) XCVESK,     'xs_crude  VESKO    ','XCVESK','  '
      WRITE(NOUT,BXL1F) XCGAUS,     'xs_crude  GAUSS    ','XCGAUS','  '
      WRITE(NOUT,BXL1F) XDEL       ,'XCVESK/XCGAUS-1    ','      ','  '
      WRITE(NOUT,BXTXT) 'Initialize KARLUD  end  '              
      WRITE(NOUT,BXCLO)         
      ELSEIF(MODE.EQ.0) THEN                
C     ==================================================================
C     ====================GENERATION====================================
C     ==================================================================
   30 CONTINUE            
C GENERATE VV             
      IF(KEYFIX.EQ.0) THEN                  
        CALL VESK1W( 0,RHOSKO,DUM1,DUM2,WTVES)           
      ELSE                
        WTVES=1D0
        VV=VVMAX          
      ENDIF               
C LOW-LEVEL MULTIPHOTON GENERATOR           
      CALL YFSGEN(VV,VVMIN,NMAX,WT1,WT2,WT3)    
!WP
      IF (WCC.EQ.0d0) GOTO 110
C--------------TESTS ON INTERNAL WEIGHTS    
C--------------DOES NOT INTERFERE WITH THE EVENT GENERATION !!! 
      REF  = VVRHO(50,SVAR,AMEL,VV,VVMIN)                   
      WTR  = REF/VVRHO(1,SVAR,AMEL,VV,VVMIN)                
      CALL VARRAN(RN,1)
      CALL GMONIT(0,IDYFS+56,WTR,1D0,RN)          
C PSEUDOREJECTION IN ORDER TO INTRODUCE REFERENCE XSECTION
      IF(KEYFIX.EQ.0 .AND. RN.GT.WTR) GOTO 110                
      WF1  = WT1*VVRHO(51,SVAR,AMEL,VV,VVMIN)/REF           
      WF2  = WT2*VVRHO(52,SVAR,AMEL,VV,VVMIN)/REF           
      WF3  = WT3          
      WF13 = WF1*WF3      
      WF123= WF1*WF2*WF3                    
      CALL GMONIT(0,IDYFS+51,WF1,  1D0,1D0)       
      CALL GMONIT(0,IDYFS+52,WF2,  1D0,1D0)       
      CALL GMONIT(0,IDYFS+53,WF3,  1D0,1D0)       
      CALL GMONIT(0,IDYFS+54,WF13, 1D0,1D0)       
      CALL GMONIT(0,IDYFS+55,WF123,1D0,1D0)       
  110 CONTINUE            
C--------------  
!WP      WT=WTVES*WT1*WT2*WT3        
      WT=WTVES*WT1*WT2*WT3 *WCC
      WTKARL=WT           
      CALL GMONIT(0,IDYFS+59,WT,  1D0,1D0)        
      CALL GF1(IDYFS+ 7,WT,1D0)           
      PAR1=WT             
      ELSE                
C     ==================================================================
C     ====================FINAL WEIGHT ANALYSIS=========================
C     ==================================================================
cc      CALL GPRINT(IDYFS+ 7)   
      CALL GMONIT(1,IDYFS+59,DUMM1,DUMM2,DUMM3)   
      WTKARL = AVERWT      
      ERKRL  = ERRELA 
      PREC   = 1D-6
      XSGS   = BREMKF(1,PREC)
      ERGS   = XSGS*PREC
      CALL VESK1W( 1,RHOSKO,XSVE,ERELVE,XCVESK)
      ERVE   = XSVE*ERELVE       
      IF(KEYFIX.EQ.1)  XCVESK=XCGAUS     
C Note that since VESK1W produces weighted events we are sending
C up the crude x-section (from VESK1W) to the calling program
      PAR1   = XCVESK         
      PAR2   = 0D0         
C NO PRINTOUT FOR MODE = 2                  
      IF(MODE.EQ.1) RETURN                  
      DDV    = XSVE/XSGS-1D0
      DDR    = ERELVE + 1D-6
      XSKR   = XCVESK*WTKARL
      ERKR   = XSKR*ERKRL
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) '     KARLUD  FINAL  REPORT     '
      WRITE(NOUT,BXTXT) '         window A              '
      WRITE(NOUT,BXL1I) NEVTOT,     'total no of events ','NEVTOT','A0'
      WRITE(NOUT,BXL1I) NEVNEG,     'WT<0        events ','NEVNEG','A1'
      WRITE(NOUT,BXL1F) XCVESK,     'xs_cru VESKO  [R]  ','XCVESK','A2'
      WRITE(NOUT,BXL2F) XSVE,ERVE,  'xs_est VESKO  [R]  ','XSVE  ','A3'
      WRITE(NOUT,BXL2F) XSGS,ERGS,  'xs_est Gauss  [R]  ','XSGS  ','A4'
      WRITE(NOUT,BXL2F) DDV,DDR,    'XCVE/XCGS-1        ','      ','A5'
      WRITE(NOUT,BXL2F) WTKARL,ERKRL,'   <WT>           ','WTKARL','A6'
      WRITE(NOUT,BXL2F) XSKR,ERKR,  'sigma_prim    [R]  ','XSKARL','A7'
      WRITE(NOUT,BXCLO)         
C     ==================================================================
C     =============SUPPL. FINAL WEIGHT ANALYSIS=========================
C     ==================================================================
      CALL GMONIT(1,IDYFS+51,DUMM1,DUMM2,DUMM3)   
      DEL1   = AVERWT-1D0                   
      DWT1   = ERRELA     
      CALL GMONIT(1,IDYFS+52,DUMM1,DUMM2,DUMM3)   
      AWF2   = AVERWT     
      DWT2   = ERRELA     
      CALL GMONIT(1,IDYFS+53,DUMM1,DUMM2,DUMM3)   
      AWF3   = AVERWT     
      DEL3   = AVERWT-GAMFA2                
      DWT3   = ERRELA     
      CALL GMONIT(1,IDYFS+54,DUMM1,DUMM2,DUMM3)   
      AWF4   = AVERWT     
      DEL4   = AVERWT-GAMFAC                
      DWT4   = ERRELA     
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) '     KARLUD  FINAL  REPORT     '
      WRITE(NOUT,BXTXT) '         window B              '
      WRITE(NOUT,BXL2F) DEL1,DWT1,  '<WF1>-1  mass wt   ','DEL1  ','B1'
      WRITE(NOUT,BXL2F) AWF2,DWT2,  '<WF2> dilat. weight','AWF2  ','B2'
      WRITE(NOUT,BXL2F) AWF3,DWT3,  '<WF3> dilat. weight','AWF3  ','B3'
      WRITE(NOUT,BXL2F) DEL3,DWT3,  '<WF3>-YGF(BETI2)   ','DEL3  ','B4'
      WRITE(NOUT,BXL2F) AWF4,DWT4,  '<WF1*WF3>          ','AWF4  ','B5'
      WRITE(NOUT,BXL2F) DEL4,DWT4,  '<WF1*WF3>-YGF(BETI)','DEL4  ','B6'
      WRITE(NOUT,BXCLO)         
C     ==================================================================
      CALL GMONIT(1,IDYFS+55,DUMM1,DUMM2,DUMM3)   
      AWF5   = AVERWT     
      DEL5   = AVERWT-GAMFAC                
      DWT5   = ERRELA     
      CALL GMONIT(1,IDYFS+56,DUMM1,DUMM2,DUMM3)   
      AWF6   = AVERWT     
      PREC = 1D-6
      XREFER = BREMKF(50,PREC)                   
      DELKAR = XREFER*AWF5/XSKR  -1D0       
      DELREF = XCVESK*AWF6/XREFER-1D0       
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) '     KARLUD  FINAL  REPORT CONT.   '
      WRITE(NOUT,BXTXT) '         WINDOW C                  '
      WRITE(NOUT,BXTXT) 'BETI= 2*ALFA/PI*(LOG(S/MEL**2)-1)       '
      WRITE(NOUT,BXTXT) 'GAMFAP= 1-PI**2*BETI**2/12              '
      WRITE(NOUT,BXTXT) 'GAMFAC=EXP(-CEULER*BETI)/GAMMA(1+BETI)  '
      WRITE(NOUT,BXTXT) 'GAMFA2=EXP(-CEULER*BETI2)/GAMMA(1+BETI2)'
      WRITE(NOUT,BXL1F)  BETI,        '                =','BETI  ','C1'
      WRITE(NOUT,BXL1F)  GAMFAP,      '                =','GAMFAP','C2'
      WRITE(NOUT,BXL1F)  GAMFAC,      '                =','GAMFAC','C3'
      WRITE(NOUT,BXL1F)  GAMFA2,      '                =','GAMFA2','C4'
      WRITE(NOUT,BXL2F) AWF5,DWT5, ' <WF1*WF3*WF4>      ','AWF5  ','C5'
      WRITE(NOUT,BXL2F) DEL5,DWT5, ' <WF1*WF3>-YGF(BETI)','DEL5  ','C6'
      WRITE(NOUT,BXTXT) 'DELKAR=XREFER*AVER(WF1*WF1*WF3)/XSKARL-1'
      WRITE(NOUT,BXTXT) 'DELREF=XCRUDE*AVER(WTR)/XREFER-1        '
      WRITE(NOUT,BXL1F) XREFER,    'reference x_sect.   ','XREFER','C7'
      WRITE(NOUT,BXL1F) DELKAR,    'XREFER*AWF5/XSKR  -1','DELKAR','C8'
      WRITE(NOUT,BXL1F) DELREF,    'XCVESK*AWF6/XREFER-1','DELREF','C9'
      WRITE(NOUT,BXCLO)         
      ENDIF               
C     =====               
      END
                 
      FUNCTION RHOSKO(R)                    
C     ********************                  
C CALLED IN VESK1W        
C PROVIDES V OR K DISTRIBUTION TO BE GENERATED                
C     ********************                  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)   
      PARAMETER (FLEPS = 1D-35)
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT     
      COMMON / VVREC  / VVMIN,VVMAX,VV,BETI                   
C MAPPING  R => VV CHANGE  TO IMPROVE ON EFFICIENCY
C Note that the replacement below makes program more precise
C and bulet-proof with respect numerical instabilities close to VV=0    
      X = MAX(R,FLEPS**BETI)                
      BBT = -0.5D0          
      CALL CHBIN1(X,BETI,BBT,VVMAX,VV,RJAC)               
C BORN XSECTION           
C NOTE 1/(1-VV) FACTOR BECAUSE BORNY IS IN R-UNITS            
      SVAR   = 4D0*ENE**2
      SVAR1  = SVAR*(1D0-VV)                 
      XBORN  = BORNY(SVAR1)/(1D0-VV)         
C constant x-section for tests 
      IF(KEYZET.EQ.-2) XBORN = XBORN * (1D0-VV)
      RHOSKO = RJAC*XBORN* VVRHO(1,SVAR,AMEL,VV,VVMIN)
      END                 

      SUBROUTINE YFSGEN(VV,VMIN,NMAX,WT1,WT2,WT3)               
C     *******************************************               
C======================================================================
C================== Y F S G E N =======================================
C======================================================================
C*********INPUT                 
C VV    = V VARIABLE            
C VMIN  = MINIMUM V VARIABLE (INFRARED CUTOFF)  
C NMAX  = MAXIMUM PHOTON MULTIPLICITY           
C*********OUTPUT                
C WT1  = WEIGHT DUE TO NEGLECTED MASS TERMS     
C WT2  = WEIGHT DUE TO DILATATION OF PHOTON MOMENTA             
C WT3  = ANOTHER DILATATION WEIGHT              
C OTHER OUTPUT RESULTS IN /MOMINI/              
C*****************************                  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      PARAMETER( PI=3.1415926535897932D0,ALFINV= 137.03604D0)
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF
      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT 
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT   
      SAVE / WEKING /,/ MOMINI /,/ KEYYFS /
      COMMON /INFORP/CCW,WCC,PHI,R(8)
      SAVE   /INFORP/
      DIMENSION XPH(100),RR(100)                
C               
C HERE BETI2 MUST BE USED INSTEAD OF BETI (MASS TERM NEGLECTED) 
      BETI2 = 2D0/ALFINV/PI*DLOG(4D0*ENE**2/AMEL**2)            
      AM2=(AMEL/ENE)**2         
      DO 10 I=1,NMAX            
      XPH(I)=0D0                
      DO 10 J=1,4               
   10 SPHOT(I,J)=0D0            
      IF(VV.LE.VMIN) THEN       
C NO PHOTON ABOVE DETECTABILITY THRESHOLD       
         WT1=1D0                
         WT2=1D0                
         WT3=1D0                
         NPHOT=0                
      ELSE      
C ONE OR MORE PHOTONS, GENERATE PHOTON MULTIPLICITY             
C NPHOT = POISSON(WITH AVERAGE = AVERG) + 1     
         AVERG=BETI2*DLOG(VV/VMIN)              
  100    CALL POISSG(AVERG,NMAX,MULTP,RR)       
         NPHOT = MULTP+1        
C This is for tests of program at fixed multiplicity (for adv. users)
         NPHFIX =  MOD(KEYBRM,10000)/1000       
         IF(NPHFIX.NE.0.AND.NPHOT.NE.NPHFIX) GOTO 100           
         IF(NPHOT.EQ.1) THEN    
            XPH(1)=VV           
            CALL BREMUL(XPH,AM2,WT1)            
            DJAC0=(1D0+1D0/SQRT(1D0-VV))/2D0    
            WT2  = 1D0/DJAC0    
            WT3  = 1D0          
         ELSE                   
            XPH(1)=VV           
            DO 200 I=2,NPHOT    
  200       XPH(I)=VV*(VMIN/VV)**RR(I-1)        
            CALL BREMUL(XPH,AM2,WT1)            
            CALL RESOLH(VV,EXPY,DJAC)           
            DJAC0=(1D0+1D0/SQRT(1D0-VV))/2D0    
            WT2  = DJAC/DJAC0   
            WT3  = 1D0          
C SCALE DOWN PHOTON ENERGIES AND MOMENTA        
            DO 300 I=1,NPHOT    
            DO 300 K=1,4        
  300       SPHOT(I,K)=SPHOT(I,K)/EXPY          
C CHECK ON LOWER ENERGY CUT-OFF                 
            IF(SPHOT(NPHOT,4).LT.VMIN) WT3 =0D0                 
         ENDIF                  
      ENDIF     
C PHOTON MOMENTA IN GEV UNITS   
      DO 420 J=1,4              
  420 SPHUM(J)=0D0              
      DO 480 I=1,NPHOT          
      DO 480 J=1,4              
      SPHOT(I,J)=SPHOT(I,J)*ENE                 
  480 SPHUM(J)=SPHUM(J)+SPHOT(I,J)              
C DEFINE FERMION MOMENTA        
      CALL KINEKR
!WP
      IF (WCC.EQ.0d0) THEN
        DO J = 1,4               
          QF1(J)=0D0
          QF2(J)=0D0
        ENDDO
        NPHOT = 0
      ENDIF
      END 
      
      SUBROUTINE RESOLH(VV,EXPY,DJAC)           
C     *******************************           
C THIS SOLVES CONSTRAINT EQUATION ON PHOTON MOMENTA             
C ALSO CALCULATES CORRESPONDING JACOBIAN FACTOR                 
C INPUT:  VV    = COSTRAINT PARAMETER V         
C OUTPUT  EXPY  = RESCALING FACTOR - A SOLUTION OF THE EQUATION 
C         DJAC  = JACOBIAN FACTOR               
C     ************************                  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION PP(4),PK(4)     
      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      SAVE   / MOMINI /
C               
      DO 210 K=1,4              
      PK(K)=0D0                 
  210 PP(K)=0D0                 
      PP(4)=2D0                 
      DO 215 I=1,NPHOT          
      DO 215 K=1,4              
  215 PK(K)=PK(K)+SPHOT(I,K)    
      PPDPP=PP(4)**2-PP(3)**2-PP(2)**2-PP(1)**2                 
      PKDPK=PK(4)**2-PK(3)**2-PK(2)**2-PK(1)**2                 
      PPDPK=PP(4)*PK(4)-PP(3)*PK(3)-PP(2)*PK(2)-PP(1)*PK(1)     
      AA=PPDPP*PKDPK/(PPDPK)**2                 
      EXPY=2D0*PPDPK/PPDPP/VV   
C SOLUTION FOR CONSTRAINT ON PHOTON FOUR MOMENTA                
      EXPY=EXPY*.5D0*(1D0+SQRT(1D0-VV*AA))      
C JACOBIAN FACTOR               
      DJAC=(1D0+1D0/SQRT(1D0-VV*AA))/2D0        
      END 
      
      SUBROUTINE BREMUL(XPH,AM2,WT)             
C     *****************************             
C PROVIDES PHOTON FOURMOMENTA   
C INPUT  : XPH    = LIST OF PHOTON ENERGIES     
C OUTPUT : SPHOT  = LIST OF PHPTON FOUR-MOMENTA                 
C          WT     = WEIGHT DUE TO MASS TERMS    
C     ************************                  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      PARAMETER( PI =3.1415926535897932D0)           
      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      SAVE   / MOMINI /
      DIMENSION XPH(*)          

      WT=1D0    
      DO 100 I=1,NPHOT          
      XK=XPH(I)                 
      CALL VARRAN(RN,1)
      CALL ANGBRE(RN,AM2,CG,SG,DIST0,DIST1)     
      WTM   =DIST1/DIST0        
      WT    =WT*WTM             
      CALL VARRAN(RNUMB,1)
      PHI=2D0*PI*RNUMB          
      SPHOT(I,1)=XK*SG*COS(PHI)                 
      SPHOT(I,2)=XK*SG*SIN(PHI)                 
      SPHOT(I,3)=XK*CG          
      SPHOT(I,4)=XK             
  100 CONTINUE                  
      END 
      
!WP      SUBROUTINE KINEKR        
!C     *****************         
!C CALLED IN YFSGEN              
!C DEFINES FINAL STATE FERMION FOUR MOMENTA      
!C     ************************                  
!      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
!      PARAMETER( PI =3.1415926535897932D0)           
!      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF
!      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT 
!      SAVE   / WEKING /,/ MOMINI /
!      DIMENSION QSU(4),DRVEC(10)          
!C               
!C QSU IS FOURMOMENTUM OF FINAL STATE FERMION PAIR               
!      DO 10 K=1,4               
!   10 QSU(K)=-SPHUM(K)          
!      QSU(4)=QSU(4)+2D0*ENE     
!      QSM =SQRT(QSU(4)**2-QSU(3)**2-QSU(2)**2-QSU(1)**2)        
!C GENERATE MOMENTA OF FERMIONS IN THEIR REST FRAME WITH FLAT DISTR.
!      QF1(4)=QSM/2D0            
!      QF2(4)=QSM/2D0            
!      QMD=SQRT(QF1(4)**2-AMFIN**2)              
!      CALL VARRAN(DRVEC,2)
!      R1 = DRVEC(1)
!      R2 = DRVEC(2)
!      COSTH=-1D0+2D0*R1         
!      SINTH=SQRT(1D0-COSTH**2)                  
!      QF1(1)=QMD*SINTH*COS(2D0*PI*R2)           
!      QF1(2)=QMD*SINTH*SIN(2D0*PI*R2)           
!      QF1(3)=QMD*COSTH          
!      DO 30 K=1,3               
!   30 QF2(K)=-QF1(K)            
!C TRANSFORM BACK TO CMS         
!      IF(NPHOT.GE.1) THEN       
!        CALL BOSTDQ(-1,QSU,QF1,QF1)             
!        CALL BOSTDQ(-1,QSU,QF2,QF2)             
!      ENDIF     
!      END 

      SUBROUTINE KINEKR
*     *****************
! New KINEKR for WW production
! Written by: Wieslaw Placzek,        Knoxville, June 1996
! Last update: 21.04.1998         by: WP 
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER ( PI=3.1415926535897932D0 )
      COMMON / Wparam / amaW,GammW
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF
      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON /KOPLUT/ V(4,5),A(4,5),B(5,5,5),XM2(5),XMG(5),CLR(4,5,2)
      COMMON /MOMENT/ PK1(5),PK2(5),P1(5),P2(5),P3(5),P4(5)
      COMMON /INFORP/CCW,WCC,PHHI,R(8)
      COMMON / PolAng / COSTH,PHI
      REAL*8 QSU(4)
      SAVE 
!
      WCC = 0D0
* QSU IS FOURMOMENTUM OF FINAL STATE FERMION PAIR
      DO 10 K=1,4
   10 QSU(K) =-SPHUM(K)
      QSU(4) = QSU(4) + 2D0*ENE
      QSM    = SQRT(QSU(4)**2-QSU(3)**2-QSU(2)**2-QSU(1)**2)
      SVAR1  = QSM**2
! Random numbers generation
      CALL RANDA(8,R,11)
! Genarate resonating pair masses
      AMG = AMAZ**2
      RMG = AMAZ*GAMMZ
      Q15 = AMG + RMG*DTAN((R(1)-0.5D0)*PI)
      Q25 = AMG + RMG*DTAN((R(2)-0.5D0)*PI)
      IF (Q15.LT.0D0) RETURN
      IF (Q25.LT.0D0) RETURN
      AM1 = DSQRT(Q15)
      AM2 = DSQRT(Q25)
      IF ((AM1+AM2).GT.QSM) RETURN
      IF (P1(5)+P2(5).GT.AM1) RETURN
      IF (P3(5)+P4(5).GT.AM2) RETURN
! Generate non-trivial polar angle
      D  = SVAR1 -Q15 -Q25
      XL = DSQRT(D*D -4D0*Q15*Q25)
      D  = D/XL
      E  = ((D+1)/(D-1))**(2*R(3)-1)
      COSTH  = D*( (E - 1)/(E + 1) )
cc      COSTH = D - (D+1D0)*( (D-1D0)/(D+1D0) )**R(3)
! Generate azimuthal angle 
      PHI = 2D0*PI*R(4) 
! CONSTRUCT MOMENTUM QF1 OF THE FIRST RESONANCE
      IF (DABS(COSTH).LT.1d0) THEN
        SINTH = SQRT(1D0 - COSTH**2)
      ELSE
cc        print*,'KINEKR: COSTH=',COSTH
        COSTH = SIGN(1D0,COSTH)
        SINTH = 0d0
      ENDIF
      QF1(4) = (SVAR1 + Q15 - Q25)/(2*QSM)
      QV     = SQRT(QF1(4)**2 - Q15)
      QF1(3) = QV*COSTH
      QF1(2) = QV*SINTH*COS(PHI)
      QF1(1) = QV*SINTH*SIN(PHI)
! CONSTRUCT THE MOMENTUM OF THE SECOND RESONANCE
      QF2(4) = QSM - QF1(4)
      DO 30 K = 1,3
   30 QF2(K) = -QF1(K)
* TRANSFORM BACK TO CMS
      IF(NPHOT.GE.1) THEN
        CALL BOSTDQ(-1,QSU,QF1,QF1)
        CALL BOSTDQ(-1,QSU,QF2,QF2)
      ENDIF
      WCC = 1D0
      END
      
      SUBROUTINE KARFIN(MODE, XXF,am0,am1,am2,WT)
!WP      SUBROUTINE KARFIN(MODE, XXF,AMFIN,WT)     
C     ***************************************   
C ======================================================================
C ... Low level Monte Carlo for final state multibremsstrahlung 
C ======================================================================
C Input:        
C         XXF    = CMS four momentum of the entire final state  
C  system i.e. fermions + photons               
C         AMFIN  = mass of final state fermion  
C Hidden input:  EPS, DELTA in /VVREC/, formats in /BXFMTS/     
C Output:       
C            WT  = crude MC weight              
C         Momenta in /MOMFIN/   
!----------------------------------------------------------------------   
! Adapted for WW process by: Wieslaw Placzek     Knoxville, June 1996
! Last update: 16.07.1996       by: WP
!----------------------------------------------------------------------   
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      DIMENSION XXF(4) 
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF  
      SAVE   / WEKING /
      COMMON / MOMFIN / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / UUREC  / UU,EPS,DELTA            
C COMMUNICATES WITH GMONIT      
      COMMON / CMONIT/ AVERWT,ERRELA,NEVTOT,NEVACC,NEVNEG,NEVOVE,NEVZER
      COMMON / INOUT  / NINP,NOUT         
      DIMENSION  MK(100), WTMAS(100)            
      COMMON / BXFMTS / BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G
      CHARACTER*80      BXOPE,BXCLO,BXTXT,BXL1I,BXL1F,BXL2F,BXL1G,BXL2G
      SAVE / MOMFIN /,/ UUREC  /,/ CMONIT/,/ INOUT  /,/ BXFMTS /
      SAVE NEVGEN,MARTOT,IDYFS
C               
      IF(MODE.EQ.-1) THEN       
C     ===================       
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) 'Initialize KARFIN  start'              
      IDYFS = 0
      CALL GBOOK1(IDYFS+20,'KARFIN: log10(k/eps) marked $',50, 0D0, 5D0)
C ...Additional check on WTCTRL, plot of its u-dependence       
      CALL GBOOK1(IDYFS+31,'KARFIN:     U  WTCTRL       $',27,.1D0, 1D0)
      CALL GBOOK1(IDYFS+32,'KARFIN:     U  WT=1         $',27,.1D0, 1D0)
      CALL GMONIT(-1,IDYFS+60,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+61,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+62,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+63,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+64,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+65,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+66,0D0,1D0,1D0)      
      CALL GMONIT(-1,IDYFS+69,0D0,1D0,1D0)      
      NEVGEN=0                  
      MARTOT=0                  
      WRITE(NOUT,BXTXT) 'Initialize KARFIN  end  '              
      WRITE(NOUT,BXCLO)         
      ELSEIF(MODE.EQ.0) THEN    
C     ======================    
      NEVGEN=NEVGEN+1           
      SVAR = XXF(4)**2-XXF(3)**2-XXF(2)**2-XXF(1)**2  
!WP      CMSENE = SQRT(SVAR) 
!WP      EMIN = EPS*CMSENE/2     
      EMIN = EPS*ENE       
      CALL MBREF(SVAR,am0,am1,am2,DELTA,              
     $           UU,NPHOT,SPHOT,SPHUM,MK,QF1,QF2,WT1,WT2,WTMAS) 
      CALL KINF1(XXF,QF1,QF2,NPHOT,SPHOT,SPHUM)
      WT3=1D0                   
      WCTRL=1D0                 
      WTREM=1D0   
      IF(WT1.NE.0.D0) THEN  
C ...optionally remove photons below epsilon from the record    
C ...and reorganize mass weights                
         CALL PIATEK(am0,am1,am2,EMIN,DELTA,WTMAS,WTREM,WT3,WCTRL) 
      ENDIF     
      XMUL= NPHOT               
      CALL GMONIT(0,IDYFS+64,XMUL,20D0,0D0)           
      CALL GMONIT(0,IDYFS+65,WCTRL,1D0,0D0)           
      CALL GF1(IDYFS+31, UU  ,WCTRL)          
      CALL GF1(IDYFS+32, UU  ,  1D0)          
      CALL GMONIT(0,IDYFS+66,WTREM,1D0,0D0)           
C ...Monitoring weights         
      XML = NPHOT               
      CALL GMONIT(0,IDYFS+60,XML,20D0,0D0)            
      CALL GMONIT(0,IDYFS+61,WT1,1D0,0D0)             
      CALL GMONIT(0,IDYFS+62,WT2,1D0,0D0)             
      CALL GMONIT(0,IDYFS+63,WT3,1D0,0D0)             
C ...marked photons             
      IF(NPHOT.GE.1) THEN       
         DO 60 I=1,NPHOT        
         XK= SPHOT(I,4)/EMIN    
         UL= LOG10(XK)          
         IF(MK(I).EQ.1)   CALL GF1(IDYFS+20,   UL,1.D0)       
         IF(MK(I).EQ.1)   MARTOT=MARTOT+1       
   60    CONTINUE               
      ENDIF     
C ...Main weight                
      WT = WT1*WT2*WT3 
      CALL GMONIT(0,IDYFS+69,WT ,1D0,0D0)             
      ELSE      
C     ====      
C ...no printout for MODE=1     
      IF(MODE.EQ.1) RETURN      
C-----------------------------------------------------------------------
C.........................Output window A...............................
      CALL GMONIT(1,IDYFS+60,AVMULT,DUMM2,DUMM3)      
      CALL GMONIT(1,IDYFS+61,AWT61,DWT61,DUMM3)       
      CALL GMONIT(1,IDYFS+62,AWT62,DWT62,DUMM3)       
      CALL GMONIT(1,IDYFS+63,AWT63,DWT63,DUMM3)       
      CALL GMONIT(1,IDYFS+69,AWT69,DWT69,DUMM3)       
C ...General information on weights             
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) 'KARFIN output - window A'              
      WRITE(NOUT,BXTXT) '    Weight Statistics   '              
      WRITE(NOUT,BXL2F) AWT69,DWT69,' general weight    ','WT    ','A1'
      WRITE(NOUT,BXL1I) NEVGEN,     ' generated events  ','NEVGEN','A2'
      WRITE(NOUT,BXL1F) AVMULT,     ' aver. ph. multi.  ','AVMULT','A3'
      WRITE(NOUT,BXL1I) MARTOT,     ' Marked photons    ','MARTOT','A4'
      WRITE(NOUT,BXL2F) AWT61,DWT61,' Kinematics, smin  ','WT1   ','A5'
      WRITE(NOUT,BXL2F) AWT62,DWT62,' Jacobian          ','WT2   ','A6'
      WRITE(NOUT,BXL2F) AWT63,DWT63,' Photon ang. dist. ','WT3   ','A7'
      WRITE(NOUT,BXCLO)         
C ...Specific details on mass weight rearrangenment, and rejection
      CALL GMONIT(1,IDYFS+64,AVMLT,DWT65,DUMM3)       
      CALL GMONIT(1,IDYFS+65,AWT65,DWT65,DUMM3)       
      CALL GMONIT(1,IDYFS+66,AWT66,DWT66,DUMM3)       
      NZER66 = NEVZER           
      NTOT66 = NEVTOT           
      WRITE(NOUT,BXOPE)         
      WRITE(NOUT,BXTXT) 'KARFIN output - window B'              
      WRITE(NOUT,BXTXT) '    on mass weights     '              
      WRITE(NOUT,BXL2F) AWT66,DWT66,' removal wgt WTREM ','WT6 ','B1'
      WRITE(NOUT,BXL1I) NTOT66,     ' no. of raw events ','    ','B2'
      WRITE(NOUT,BXL1I) NZER66,     ' WT6=0      events ','    ','B3'
      WRITE(NOUT,BXL1F) AVMLT,      ' raw ph. multipl.  ','    ','B4'
      WRITE(NOUT,BXL2F) AWT65,DWT65,' control wgt WCTRL ','WT5 ','B5'
      WRITE(NOUT,BXL1G) EPS,        ' epsilon           ','    ','B6'
      WRITE(NOUT,BXL1G) DELTA,      ' delta             ','    ','B7'
      WRITE(NOUT,BXCLO)         
C ...Histograms                 
cc      CALL GPRINT(IDYFS+20)         
cc      CALL GOPERA(IDYFS+31,'/',IDYFS+32,IDYFS+33,1D0,1D0)   
cc      CALL GPRINT(IDYFS+33)         
      ENDIF     
C     =====     
      END 

      SUBROUTINE PIATEK(am0,am1,am2,EMIN,DELTA,WTMAS,WTREM,WTM3A,WCTRL)      
!WP      SUBROUTINE PIATEK(AMFIN,EMIN,DELTA,WTMAS,WTREM,WTM3A,WCTRL)
C     ***********************************************************
C Written CERN piatek 22 sept. 89  (S.J.)       
C Note the action of this routine is not Loretnz invariant !!!! 
C         KEYPIA   =0,1 REMOVAL OF PHOTONS BELOW EMIN OFF/ON    
C INPUT:  momenta in / MOMFIN /                 
C         AMFIN    = fermion mass (GEV)         
C         EMIN     = Emin minimum energy of photons to be left (GEV)
C         DELTA    = infrared cut-off in generation (dimensionless)
C         WTMAS    = list of mass weights for all photons       
C OUTPUT: WTREM    = 1 for KEYPIA=0             
C  = mass weight of removed photons for KEYPIA=1                
C         WTM3A    = mass weight for all photons, see comments below
C         WCTRL    = control weight for removed photons
!----------------------------------------------------------------------   
! Adapted for WW process by: Wieslaw Placzek     Knoxville, June 1996
! Last update: 16.07.1996       by: WP
!----------------------------------------------------------------------       
C     ***********************************       
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      PARAMETER(PI=3.1415926535897932D0,ALFINV=137.03604D0)     
      DIMENSION  WTMAS(100)     
      COMMON / MOMFIN / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT      
      COMMON / INOUT  / NINP,NOUT  
      SAVE / MOMFIN /,/ KEYYFS /,/ INOUT  /
      DIMENSION QQ(4),PP(4)

      KEYPIA = KEYRED 
      DO 10 K=1,4               
      PP(K) = QF1(K)+QF2(K) +SPHUM(K)           
  10  QQ(K) = QF1(K)+QF2(K)     
      SVAR = PP(4)**2-PP(3)**2 -PP(2)**2 -PP(1)**2              
      SPRIM= QQ(4)**2-QQ(3)**2 -QQ(2)**2 -QQ(1)**2  
!WP------------------------------------------------
      AMFIN = am0
      bt0 = SQRT(1 - 4*am0**2/SVAR)
      bb0 = (1 + bt0**2)/2/bt0
      xmu  = qf1(4)*qf2(4) -qf1(3)*qf2(3) -qf1(2)*qf2(2) -qf1(1)*qf2(1)
      rho  = SQRT( 1 - (am1*am2/xmu)**2 )
      blg0 = LOG( (1+bt0)/(1-bt0) ) *bb0
      blg1 = LOG( xmu*(1+rho)/am1/am2 ) /rho
!--------------------------------------------------
      WTM1=1D0                  
      WTM2=1D0                  
C ...mass weight below and above epsilon calculated separately  
      DO 40 I=1,NPHOT           
      IF(SPHOT(I,4).LT.EMIN) THEN               
         WTM1=WTM1*WTMAS(I)     
         IF(WTM1.LE.1D-30) WTM1=0D0             
      ELSE      
         WTM2=WTM2*WTMAS(I)     
         IF(WTM2.LE.1D-30) WTM2=0D0             
      ENDIF     
   40 CONTINUE                  
C ...All sort of weights        
      ALF1  =  1/PI/ALFINV      
      QQK  = QQ(4)*SPHUM(4)-QQ(3)*SPHUM(3)-QQ(2)*SPHUM(2)-QQ(1)*SPHUM(1)
C ...delt1 and eps1 are cutoffs which sit in YFS formfactor     
!WP      DELT1 =  DELTA*(1+ 2*QQK/SPRIM)
!WP---------------------------------------------------------------
      AMK2 = SPHUM(4)**2 - SPHUM(3)**2 - SPHUM(3)**2 - SPHUM(1)**2
      DELT1 = DELTA*SQRT(1 + (2*QQK+AMK2)/SPRIM)
!-----------------------------------------------------------------
      EPS1  =  SQRT(EMIN**2/QF1(4)/QF2(4)) 
C ...YFS formfactor cut-off dependend part: delt1 = 2Emin/sqrt(s')
C ...where Emin is infrared cut-off in QMS      
!WP      FYFS  = -2*ALF1*(DLOG(SPRIM/AMFIN**2)-1)*DLOG(1/DELT1)
      FYFS = -2*ALF1*(blg1 - 1)*LOG(1/DELT1)
C ...The total phase space integral for crude x-section,        
C ...Note that delta is a lower limit on y-variables used in generation
!WP      FPHS  =  2*ALF1* DLOG(SVAR/AMFIN**2) *DLOG(1/DELTA)
      FPHS = 2*ALF1*blg0*LOG(1/DELTA)       
      DELB  =  FYFS + FPHS      
C ...The average mass weight for removed photon = EXP(DELB2)    
C ...Can be calculated analytically as a  ratio of YFS formfactors
C ...On the other hand it is checked by MC, see control weight WCTRL
!WP      DELB2 = -2*ALF1*(DLOG(SVAR/SPRIM)+1) *DLOG(EPS1/DELT1) 
      DELB2 = -2*ALF1* (blg0 - blg1 + 1) *LOG(EPS1/DELT1)   
C ...Control weight - its average should be precisely one       
      WCTRL =WTM1*EXP(-DELB2)   
      IF(KEYPIA.EQ.0) THEN      
         IF(ABS(DELB).GT.100D0 ) WRITE(NOUT,*) ' DELB= ',DELB  
         IF(ABS(DELB).GT.100D0 ) WRITE(   6,*) ' DELB= ',DELB  
         WTREM = 1D0            
         WTM3A = WTM1*WTM2*EXP(DELB)            
      ELSE      
C ...Optional removal of photons below epsilon from the record  
C ...in such a case WTM3A includes exp(DELB2)= <WT3> for removed ph.
         NPH=NPHOT              
         DO 100 J=NPHOT,1,-1    
         IF(SPHOT(J,4).LT.EMIN) THEN 
            DO 60 I=J+1,NPH     
            DO 60 K=1,4         
   60       SPHOT(I-1,K)=SPHOT(I,K)             
            NPH=NPH-1           
         ENDIF                  
  100    CONTINUE               
         NPHOT=NPH              
C ...WTMAS includes here average weight of removed photons exp(DELB2)
         WTREM = WTM1           
         WTM3A = WTM2*EXP(DELB+DELB2)           
      ENDIF     
      END 
      
      SUBROUTINE WFORM(CMSENE,Q1,Q2,AMF,DELTA,EPS,DYFS)         
C     *************************************************         
C Not used, kept for some future tests          
C Yennie-Fraytschi-Suura Formfactors for the final state ferm. pair
C     ***********************************       
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)       
      PARAMETER(PI=3.1415926535897932D0,ALFINV=137.03604D0)     
      PARAMETER(ALF1=1/ALFINV/PI)         
      DIMENSION Q1(4),Q2(4)     

      SVAR  = CMSENE**2         
      ENE= CMSENE/2             
C ...Momenta q1,q2 should be in CMS             
      E1 = Q1(4)                
      E2 = Q2(4)                
      BETF2 = 2*ALF1* DLOG(SVAR /AMF**2)        
      DELB  = BETF2*DLOG(ENE/SQRT(E1*E2)*EPS/DELTA)             
      EP    = E1+E2             
      EM    = E1-E2             
      Q1Q2  = Q1(4)*Q2(4)-Q1(3)*Q2(3)-Q1(2)*Q2(2)-Q1(1)*Q2(1)   
      DL    = SQRT( 2*Q1Q2 +EM**2 )             
      REMN  = PI**2/2           
     $        -0.50*DLOG(E1/E2)**2              
     $        -0.25*DLOG((DL+EM)**2/(4*E1*E2))**2               
     $        -0.25*DLOG((DL-EM)**2/(4*E1*E2))**2               
     $        - DILOGY((DL+EP)/(DL+EM)) -DILOGY((DL-EP)/(DL-EM))  
     $        - DILOGY((DL-EP)/(DL+EM)) -DILOGY((DL+EP)/(DL-EM))  
      DYFS  = EXP( DELB +ALF1*REMN )            
      END 

      SUBROUTINE MBREF(SVAR,xm0,xm1,xm2,DELTA,        
     $                 UU,NPHOT,PHOT,PHSU,MK,Q1,Q2,WT1,WT2,WTM)       
!WP      SUBROUTINE MBREF(SVAR,AMFIN,DELTA,
!WP     $                 UU,NPHOT,PHOT,PHSU,MK,Q1,Q2,WT1,WT2,WTM) 
C     *****************************************************************
C SIMULATES FINAL STATE BREMSSTRAHLUNG (11 MARCH 1989)         
C INPUT  : SVAR   = S VARIABLE  (GEV)         
C          AMFIN  = MASS OF FINAL FERMION     
C          DELTA  = LOWER ENERGY BOUND (DIMENSIONLESS)         
C OUTPUT : UU     = 1-s'/s   
C          NPHOT  = PHOTON MULTIPLICITY       
C          PHOT   = PHOTON FOUR MOMENTA (GEV) IN CMS           
C          PHSU   = SUM OF PHOTON MOMENTA     
C          MK     = MARKS ON PHOTONS CLOSE TO LOWER ENERGY BOUND 
C          Q1     = FINAL FERMION FOUR MOMENTUM (GEV)          
C          Q2     = FINAL FERMION FOUR MOMENTUM                
C          WT1    = THE WEIGHT - PHASE SPACE LIMITS FOR VERY HARD PHOT.
C          WT2    = THE WEIGHT - TRANSLATION JACOBIAN.         
C          WTM    = The list of mass weights. 
!----------------------------------------------------------------------   
! Adapted for WW process by: Wieslaw Placzek     Knoxville, June 1996
! Last update: 16.07.1996       by: WP
!----------------------------------------------------------------------    
C     **************************************  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)     
      PARAMETER(PI=3.1415926535897932D0,ALFINV=137.03604D0)    
      DIMENSION PHOT(100,4),PHSU(4),MK(100),WTM(100),Q1(4),Q2(4)
      DIMENSION RR(100),XK(100),CGX(100),SGX(100)              
      COMMON / INOUT  / NINP,NOUT       
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT
      SAVE / INOUT  /,/ KEYYFS /

C KEYCMS =0 INHIBITS TRANSFORMATION TO CMS    
C INITIALIZATIONS            
      UU= -1D0               
      WT1=1.D0               
      WT2=1.D0               
      AMFIN = xm0
      AM2= 4.D0*AMFIN**2/SVAR                 
      DO 10 I=1,100          
      RR(I)=0.D0             
   10 MK(I)=0                
      DO 20 K=1,4            
      PHSU(K)=0.D0           
      Q1(K)=0.D0             
   20 Q2(K)=0.D0             
      DO 21 I=1,100          
      DO 21 K=1,4            
   21 PHOT(I,K)=0.D0         
C GENERATE PHOTON MULTIPLICITY, AVERG = AVERAGE MULTIPLICITY (CRUDE)
!WP      BETF2 = 2/PI/ALFINV*DLOG(SVAR/AMFIN**2)
!WP----------------------------------------------------                  
      bt0 = SQRT(1 - AM2)
      bb0 = (1 + bt0**2)/2/bt0
      BETF2 = 2/PI/ALFINV *bb0*LOG( (1+bt0)/(1-bt0) )
!------------------------------------------------------
      AVERG = BETF2*DLOG(1/DELTA)             
      NMAX=50                
    5 CONTINUE               
      CALL POISSG(AVERG,NMAX,NPHOT,RR) 
C This is for tests of program at fixed multiplicity (for adv. users)
      NPHFIX =  MOD(KEYBRM,100000)/10000      
      IF(NPHFIX.NE.0.AND.NPHOT.NE.NPHFIX) GOTO 5               
      IF(NPHOT.EQ.0) THEN    
          SPRIM=SVAR 
          xm1s = xm1**2
          xm2s = xm2**2  
          Ene1 = (SPRIM + xm1s - xm2s)/2/SQRT(SPRIM)
          qmod = SQRT(Ene1**2 - xm1s)
          Q1(3) = qmod
          Q1(4) = Ene1
          Q2(3) =-qmod
          Q2(4) = SQRT(SPRIM) - Ene1
      ELSE                   
C BEGIN WITH PHOTON ENERGY   
          XSUM=0.D0          
          DO  50 I=1,NPHOT   
          XK(I)=DELTA**RR(I)                  
          IF(XK(I).LT.SQRT(10.D0)*DELTA) MK(I)=1               
   50     XSUM=XSUM+XK(I)    
          IF(XSUM.GE.1.D0) GOTO 900           
          DO 60 I=1,NPHOT    
   60     XK(I)=XK(I)/(1.D0-XSUM)             
          DO 100 I=1,NPHOT   
C ...Photons close to low energy boundry marked for further control
          CALL VARRAN(RN1,1)
          CALL VARRAN(RN2,1)
C ...Simplified photon angular distribution,  
C ...s'->s and m**2/(kp)**2 dropped           
C ...CG=cos(theta) and SG=sin(theta) memorized to avoid rounding err.
!WP          CALL ANGBRE(RN1,AM2,CG,SG,WTM(I),DIS1)
!WP----------------------------------------------------   
          CALL ANGBRE(RN1,AM2,CG,SG,DIS0,DIS1)  
          WTM(I) = DIS0 *2*(1+bt0**2)
!------------------------------------------------------
          PHI=2.D0*PI*RN2    
C ...Define photon mementa (in units of sqrt(s')/2 )           
          PHOT(I,1)=XK(I)*SG*COS(PHI)         
          PHOT(I,2)=XK(I)*SG*SIN(PHI)         
          PHOT(I,3)=XK(I)*CG                  
          PHOT(I,4)=XK(I)    
          DO 70 K=1,4        
   70     PHSU(K)=PHSU(K)+PHOT(I,K)           
          CGX(I)=CG          
          SGX(I)=SG          
  100     CONTINUE           
C ...Determine rescaling factor and s', WT2 is dilatation Jacobian 
          XMK2 = PHSU(4)**2-PHSU(3)**2-PHSU(2)**2-PHSU(1)**2   
          YY   = 1.D0/(1.D0 +PHSU(4) +XMK2/4.D0 )              
          WT2  = YY*(1.D0+PHSU(4))            
          SPRIM= SVAR*YY   
C ...reject events with too hard photons      
!WP          SMINI= 4*AMFIN**2  
          SMINI = (xm1 + xm2)**2
          IF(SPRIM.LE.SMINI) GOTO 900  
C ...rescale properly all photon momenta      
          ENER = SQRT(SPRIM)/2.D0             
          DO 120 K=1,4       
          PHSU(K)= PHSU(K)*ENER               
          DO 120 I=1,NPHOT   
  120     PHOT(I,K)=PHOT(I,K)*ENER 
!WP---------------------------------------------------------
! Final particle 4-momenta
          xm1s = xm1**2
          xm2s = xm2**2  
          Ene1 = (SPRIM + xm1s - xm2s)/2/SQRT(SPRIM)
          qmod = SQRT(Ene1**2 - xm1s)
          Q1(3) = qmod
          Q1(4) = Ene1
          Q2(3) =-qmod
          Q2(4) = SQRT(SPRIM) - Ene1
!------------------------------------------------------------           
C-----------Mass weight---------------------  
C ...This weight compensates for s->s' and droping terms -m**2/(k.q)**2 
C ...care is taken of machine rounding errors                  
!WP          AMF= 4.D0*AMFIN**2/SPRIM            
!WP          BTF =SQRT(1.D0-AMF)
!WP-------------------------------------------------------------------- 
          rm1 = ( xm1/Q1(4) )**2
          rm2 = ( xm2/Q2(4) )**2
          bt1 = SQRT(1 - rm1)
          bt2 = SQRT(1 - rm2)
          rq12 = 2*(1 + bt1*bt2)
!----------------------------------------------------------------------
          DO 200 I=1,NPHOT   
!WP          IF( CGX(I).GT.0.D0 ) THEN           
!WP              DEL2=1.D0+BTF*CGX(I)            
!WP              DEL1= (SGX(I)**2 + AMF*CGX(I)**2)/DEL2           
!WP          ELSE               
!WP              DEL1=1.D0-BTF*CGX(I)            
!WP              DEL2= (SGX(I)**2 + AMF*CGX(I)**2)/DEL1           
!WP          ENDIF              
!WP          DIST1=1/DEL1/DEL2                   
!WP     $            -AMF/(1.D0+BTF )/2.D0*(1.D0/DEL1**2+1.D0/DEL2**2)
!WP--------------------------------------------------------------------
             del1 = 1 - bt1*CGX(I)
             del2 = 1 + bt2*CGX(I)
             rdel = del1/del2
             DIST1 = rq12/del1/del2 *( 1 - (rm1/rdel + rm2*rdel)/rq12 )
!----------------------------------------------------------------------
          IF(DIST1.LT.0.D0) THEN              
              DIST1=0D0 
              WRITE(NOUT,*) ' +++++++ MBREF: NEGATIVE WTMAS =',DIST1
              WRITE(   6,*) ' +++++++ MBREF: NEGATIVE WTMAS =',DIST1
          ENDIF              
          WTM(I)= DIST1/WTM(I)                
          IF(WTM(I).LT. 1.D-20) WTM(I)= 0.D0  
  200     CONTINUE           
      ENDIF   
      UU   = 1 - SPRIM/SVAR                   
!WP      ENER= SQRT(SPRIM)/2.D0                  
!WPC FINAL FERMION MOMENTA      
!WP      Q1(4)=ENER             
!WP      Q1(3)=SQRT(ENER**2-AMFIN**2)            
!WP      Q2(4)=ENER             
!WP      Q2(3)=-Q1(3)
      RETURN                 
C EVENT OUTSIDE PHASE SPACE                   
  900 WT1=0.D0               
      WT2=1.D0               
      NPHOT=-1               
      END 
                   
      SUBROUTINE KINF1(XXF,Q1,Q2,NPHOT,PHOT,PHSU)              
C     *******************************************              
C Transforms to CMS: PHOT, PHSU, Q1,Q2        
C with random Euler rotation in XXF frame (Z frame)            
C     ***********************************     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)     
      PARAMETER( PI = 3.1415926535897932D0)
!WP: Polar angle from KINEKR
      COMMON / PolAng / COSTH,PHIX               
      DIMENSION XXF(4),PHOT(100,4),PHSU(4),Q1(4),Q2(4)         
      DIMENSION PH(4),PP(4),DRVEC(10)                   

      IF(NPHOT.EQ.-1) RETURN                  
      CALL VARRAN(DRVEC,2)
c      RN1 = DRVEC(1)
c      RN2 = DRVEC(2)
!WP      CTH= 1.D0 -2.D0*RN1 
      CTH = COSTH
      THE= ACOS(CTH)         
!WP      PHI= 2.D0*PI*RN2
      PHI = PHIX
      DO 210 K=1,4           
  210 PP(K)=Q1(K)+Q2(K)+PHSU(K)               
      DO 230 I=1,NPHOT       
          DO 220 K=1,4       
  220     PH(K)=PHOT(I,K)    
          CALL BOSTDQ(  1, PP,PH,PH)          
          CALL ROTEUL(THE,PHI,PH,PH)          
          CALL BOSTDQ( -1,XXF,PH,PH)          
          DO 225 K=1,4       
  225     PHOT(I,K)= PH(K)   
  230 CONTINUE               
      CALL BOSTDQ(  1, PP,Q1,Q1)              
      CALL ROTEUL(THE,PHI,Q1,Q1)              
      CALL BOSTDQ( -1,XXF,Q1,Q1)              
      CALL BOSTDQ(  1, PP,Q2,Q2)              
      CALL ROTEUL(THE,PHI,Q2,Q2)              
      CALL BOSTDQ( -1,XXF,Q2,Q2)              
      CALL BOSTDQ(  1, PP,PHSU,PHSU)          
      CALL ROTEUL(THE,PHI,PHSU,PHSU)          
      CALL BOSTDQ( -1,XXF,PHSU,PHSU)          
      END  
                  
      SUBROUTINE ROTEUL(THE,PHI,PVEC,QVEC)    
C     ************************************    
C EULER ROTATION             
      IMPLICIT REAL*8(A-H,O-Z)                
      REAL*8 PVEC(4),QVEC(4)                  
      CALL ROTOD1(THE,PVEC,QVEC)              
      CALL ROTOD3(PHI,QVEC,QVEC)              
      END 
                   
      SUBROUTINE GIBEA(CMSENE,AMEL,P1,P2)     
C     ***********************************     
C GIVEN CMS ENERGY (CMSENE) DEFINES BEAM MOMENTA IN CMS        
C     ***********************************     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)     
      DIMENSION P1(*),P2(*)                   
      BETEL=SQRT(1D0-4D0*AMEL**2/CMSENE**2)   
      P1(1)=  0D0            
      P1(2)=  0D0            
      P1(3)=  CMSENE/2D0*BETEL                
      P1(4)=  CMSENE/2D0     
      P2(1)=  0D0            
      P2(2)=  0D0            
      P2(3)= -CMSENE/2D0*BETEL                
      P2(4)=  CMSENE/2D0     
      END 
                   
      SUBROUTINE KINF2(XXF,AMFIN,Q1,Q2)       
C     ************************************    
C ...Generates two body phase space with uniform spherical density 
C ...For pure Born case (no bremss.)          
C     ***********************************     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)     
      PARAMETER( PI = 3.1415926535897932D0)               
      DIMENSION XXF(*),Q1(*),Q2(*),DRVEC(10)            

      CMSENE= SQRT(XXF(4)**2-XXF(3)**2-XXF(2)**2-XXF(1)**2)    
      CALL VARRAN(DRVEC,2)
      RN1 = DRVEC(1)
      RN2 = DRVEC(2)
      CTH= 1.D0 -2.D0*RN1    
      THE= ACOS(CTH)         
      PHI= 2.D0*PI*RN2       
      CALL GIBEA(CMSENE,AMFIN,Q1,Q2)          
      CALL ROTEUL(THE,PHI,Q1,Q1)              
      CALL BOSTDQ( -1,XXF,Q1,Q1)              
      CALL ROTEUL(THE,PHI,Q2,Q2)              
      CALL BOSTDQ( -1,XXF,Q2,Q2)              
C======================================================================
C=====================End of KARFIN part===============================
C======================================================================
      END
                    
      FUNCTION BORNY(SVAR1)     
C     ********************      
C THIS ROUTINE CALCULATES TOTAL BORN CROSS SECTION.             
C IT EXPLOITS THE FACT THAT BORN X. SECTION = A + B*C + D*C**2  
C     ********************      
      IMPLICIT DOUBLE PRECISION  (A-H,O-Z) 
!WP      SVAR=SVAR1                
!WP      F0=BORNV(SVAR, 0.D0)      
!WP      BORNY= F0 
      BORNY = 1d0
      END       

      FUNCTION BORNVC(SVARI,COSTHE)     
!WP      FUNCTION BORNV(SVARI,COSTHE)            
C     ***********************************     
C THIS ROUTINE PROVIDES BORN DIFFERENTIAL CROSS SECTION        
C a version without COMPLEX*16                
C     ***********************************     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)     
      COMMON / INOUT  / NINP,NOUT       
      COMMON / WEKING / ENE,AMAZ,GAMMZ,AMEL,AMFIN,XK0,SINW2,IDE,IDF 
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT     
      SAVE / INOUT  /,/ WEKING /,/ KEYYFS /
C           
      IF(ABS(COSTHE).GT.1D0) WRITE(NOUT,*) ' BORNV: COSTHE=',COSTHE
      QE= -1D0               
      QF= -1D0               
      AA= 4D0*SQRT(SINW2*(1D0-SINW2))         
      VE= (-1D0+4*SINW2)/AA                   
      AE= 1D0/AA             
      IF(KEYZET.LE.0) THEN   
        VE=0D0               
        AE=0D0               
      ENDIF                  
      VF= (-1D0+4*SINW2)/AA                   
      AF= 1D0/AA             
      S = SVARI              
      CHI2 =         S**2/((S-AMAZ**2)**2+(GAMMZ*AMAZ)**2)     
      RECHI=(S-AMAZ**2)*S/((S-AMAZ**2)**2+(GAMMZ*AMAZ)**2)     
      XE= VE**2 +AE**2       
      XF= VF**2 +AF**2       
      YE= 2*VE*AE            
      YF= 2*VF*AF            
      FF0= QE**2*QF**2 +2*RECHI*QE*QF*VE*VF +CHI2*XE*XF        
      FF1=             +2*RECHI*QE*QF*AE*AF +CHI2*YE*YF        
      BORN    = (1D0+ COSTHE**2)*FF0 +2D0*COSTHE*FF1           
C************                
C THIS IS A BIT CRUDE METHOD OF INTRODUCING THRESHOLD BEHAVIOUR
      IF(    SVARI.LE. 4D0*AMFIN**2) THEN     
        THRESH=0D0           
      ELSEIF(SVARI.LE.16D0*AMFIN**2) THEN     
        AMX2=4D0*AMFIN**2/SVARI               
        THRESH=SQRT(1D0-AMX2)*(1D0+AMX2/2D0)  
      ELSE                   
        THRESH=1D0           
      ENDIF                  
      BORNV= BORN*THRESH  
!WP
      BORNVC = BORNV
      END                         

      SUBROUTINE POISSG(AVERG,NMAX,MULT,RR)
C     ************************************** 
C Last corr. Nov. 91              
C This generates photon multipl. NPHOT according to Poisson distr. 
C INPUT:  AVERG = AVERAGE MULTIPLICITY   
C         NMAX  = MAXIMUM MULTIPLICITY   
C OUTPUT: MULT = GENERATED MULTIPLICITY 
C         RR(1:100) LIST OF ORDERED UNIFORM RANDOM NUMBERS, 
C         A BYPRODUCT RESULT, TO BE EVENTUALLY USED FOR SOME FURTHER
C         PURPOSE (I.E.  GENERATION OF PHOTON ENERGIES). 
C     ************************           
      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
      DIMENSION RR(*)                    
      COMMON / INOUT  / NINP,NOUT  
      SAVE   / INOUT  /
      SAVE NFAIL
      DATA NFAIL/0/                      
   50 NN=0                               
      SUM=0D0                            
      DO 100 IT=1,NMAX                   
      CALL VARRAN(RN,1)
      Y= LOG(RN)                         
      SUM=SUM+Y                          
      NN=NN+1                            
      RR(NN)=SUM/(-AVERG)                
      IF(SUM.LT.-AVERG) GOTO 130         
  100 CONTINUE                           
      NFAIL=NFAIL+1                      
      IF(NFAIL.GT.100) GOTO 900          
      GOTO 50                            
  130 MULT=NN-1                         
      RETURN                             
  900 WRITE(NOUT,*) ' POISSG: TO SMALL NMAX'
      STOP                               
      END                                

      SUBROUTINE ANGBRE(RN1,AM2,COSTHG,SINTHG,DIST0,DIST1)
C     **************************************************** 
C THIS ROUTINE GENERATES PHOTON ANGULAR DISTRIBUTION 
C IN THE REST FRAME OF THE FERMION PAIR. 
C THE DISTRIBUTION IS TAKEN IN THE INFRARED LIMIT.
C GENERATES WEIGHTED EVENTS              
C INPUT:  AM2 = 4*MASSF**2/S WHERE MASSF IS FERMION MASS
C         AND S IS FERMION PAIR EFFECTIVE MASS.
C OUTPUT: COSTHG, SINTHG, COS AND SIN OF THE PHOTON 
C         ANGLE WITH RESPECT TO FERMIONS DIRECTION 
C         DIST0 = distribution  generated without m**2/(kp)**2 terms
C         DIST1 = distribution  with m**2/(kp)**2 terms 
C     *************************************** 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
      BETA=SQRT(1.D0-AM2)                
      EPS=AM2/(1.D0+BETA)      
      DEL1=(2.D0-EPS)*(EPS/(2.D0-EPS))**RN1 
      DEL2=2.D0-DEL1                     
C SYMMETRIZATION                         
      CALL VARRAN(RN2,1)
      IF(RN2.LE.0.5D0) THEN              
        A=DEL1                           
        DEL1=DEL2                        
        DEL2=A                           
      ENDIF                              
      DIST0=1D0/DEL1/DEL2                
      DIST1=DIST0-EPS/2.D0*(1D0/DEL1**2+1D0/DEL2**2)
C CALCULATION OF SIN AND COS THETA FROM INTERNAL VARIABLES 
      COSTHG=(1.D0-DEL1)/BETA            
      SINTHG=SQRT(DEL1*DEL2-AM2)/BETA    
      END                                

      SUBROUTINE DUMPS(NOUT)                 
C     **********************                 
C THIS PRINTS OUT FOUR MOMENTA OF PHOTONS    
C ON UNIT NO. NOUT                           
C     **********************                 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)    
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT  
      SAVE   / MOMSET /
      DIMENSION SUM(4)                       
      WRITE(NOUT,*) '=====================DUMPS===================='
      WRITE(NOUT,3100) 'Q1 ',(  QF1(  K),K=1,4) 
      WRITE(NOUT,3100) 'Q2 ',(  QF2(  K),K=1,4) 
      DO 100 I=1,NPHOT                       
  100 WRITE(NOUT,3100) 'PHO',(SPHOT(I,K),K=1,4) 
      DO 200 K=1,4                           
  200 SUM(K)=QF1(K)+QF2(K)                   
      DO 210 I=1,NPHOT                       
      DO 210 K=1,4                           
  210 SUM(K)=SUM(K)+SPHOT(I,K)               
      WRITE(NOUT,3100) 'SUM',(  SUM(  K),K=1,4) 
 3100 FORMAT(1X,A3,1X,5F18.12)               
      END                                    

      SUBROUTINE DUMPF(NOUT)                 
C     **********************                 
C THIS PRINTS OUT FOUR MOMENTA OF PHOTONS    
C ON UNIT NO. NOUT                           
C     **********************                 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)    
      COMMON / MOMFIN / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      SAVE   / MOMFIN /
      DIMENSION SUM(4)                       
      WRITE(NOUT,*) '=====================DUMPF====================' 
      WRITE(NOUT,3100) 'QF1',(  QF1(  K),K=1,4) 
      WRITE(NOUT,3100) 'QF2',(  QF2(  K),K=1,4)  
      DO 100 I=1,NPHOT                       
  100 WRITE(NOUT,3100) 'PHO',(SPHOT(I,K),K=1,4) 
      DO 200 K=1,4                           
  200 SUM(K)=QF1(K)+QF2(K)                   
      DO 210 I=1,NPHOT                       
      DO 210 K=1,4                           
  210 SUM(K)=SUM(K)+SPHOT(I,K)               
      WRITE(NOUT,3100) 'SUM',(  SUM(  K),K=1,4) 
 3100 FORMAT(1X,A3,1X,5F18.14)               
      END                                    

      SUBROUTINE DUMPI(NOUT)                 
C     **********************                 
C THIS PRINTS OUT FOUR MOMENTA OF PHOTONS    
C ON UNIT NO. NOUT                           
C     **********************                 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)    
      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT 
      SAVE   / MOMINI /
      DIMENSION SUM(4)                       
      WRITE(NOUT,*) '=====================DUMPI===================='
      WRITE(NOUT,3100) 'QF1',(  QF1(  K),K=1,4) 
      WRITE(NOUT,3100) 'QF2',(  QF2(  K),K=1,4) 
      DO 100 I=1,NPHOT                       
  100 WRITE(NOUT,3100) 'PHO',(SPHOT(I,K),K=1,4) 
      DO 200 K=1,4                           
  200 SUM(K)=QF1(K)+QF2(K)                   
      DO 210 I=1,NPHOT                       
      DO 210 K=1,4                           
  210 SUM(K)=SUM(K)+SPHOT(I,K)               
      WRITE(NOUT,3100) 'SUM',(  SUM(  K),K=1,4) 
 3100 FORMAT(1X,A3,1X,5F18.14)               
      END                                    

      SUBROUTINE DUMPN(NOUT,IEV)                 
C     **************************                 
C THIS PRINTS OUT FOUR MOMENTA OF final state     
C and the serial number of event IEV on unit NOUT
C     **********************                 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)    
      COMMON / MOMINI / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT 
      SAVE   / MOMINI /
      DIMENSION SUM(4)                       
      WRITE(NOUT,*) '=============DUMPNEW====================>',IEV
      WRITE(NOUT,3100) 'QF1',(  QF1(  K),K=1,4) 
      WRITE(NOUT,3100) 'QF2',(  QF2(  K),K=1,4) 
      DO 100 I=1,NPHOT                       
  100 WRITE(NOUT,3100) 'PHO',(SPHOT(I,K),K=1,4) 
      DO 200 K=1,4                           
  200 SUM(K)=QF1(K)+QF2(K)                   
      DO 210 I=1,NPHOT                       
      DO 210 K=1,4                           
  210 SUM(K)=SUM(K)+SPHOT(I,K)               
      WRITE(NOUT,3100) 'SUM',(  SUM(  K),K=1,4) 
 3100 FORMAT(1X,A3,1X,5F18.14)               
      END   

      subroutine KinDec 
*     *****************
!----------------------------------------------------------------------!
! This routine calculates the kinematics of W+W- decays, i.e.          !
! provides 4-momenta of final state fermions (products of W-decays).   !
! The final state fermions 4-momenta are located in:                   !
!     COMMON / MOMDEC / pf1(4),pf2(4),pf3(4),pf4(4),phot(100,4),nphot  !
! where pf1, pf2 are 4-momenta of W- decay products, while pf3, pf4    !
! are 4-momenta of W+ decay products.                                  !
!----------------------------------------------------------------------!
! Written by:  Wieslaw Placzek                    Knoxville, July 1996 !  
! Last update: 22.10.1999                by: W.P.                      !
!----------------------------------------------------------------------! 
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER ( pi = 3.1415926535897932D0 )
      COMMON /INFORP/CCW,WCC,PHHI,R(8)
      COMMON /MOMENT/ PK1(5),PK2(5),P1(5),P2(5),P3(5),P4(5)
      COMMON / WGTALL / WTMOD,WTCRU1,WTCRU2,WTSET(100)  
      COMMON / MOMSET / q1(4),q2(4),phsum(4),sphot(100,4),nsphot
      COMMON / MOMDEC / pf1(4),pf2(4),pf3(4),pf4(4),phot(100,4),nphot
      SAVE   /INFORP/,/MOMENT/,/ WGTALL /,/ MOMSET /,/ MOMDEC / 
      REAL*8 h(4)
!
      DO k = 1,4
        pf1(k) = 0d0
        pf2(k) = 0d0
        pf3(k) = 0d0
        pf4(k) = 0d0
      ENDDO
      IF (WTCRU1*WTCRU2.EQ.0d0) RETURN
      sv1 = q1(4)**2 - q1(3)**2 - q1(2)**2 - q1(1)**2
      sv2 = q2(4)**2 - q2(3)**2 - q2(2)**2 - q2(1)**2
      xm1 = SQRT(sv1)
      xm2 = SQRT(sv2)
! Angular variables
      f1 = 2*PI*R(5)
      f2 = 2*PI*R(6)
      c1 = 2*R(7) - 1d0
      c2 = 2*R(8) - 1d0
! Construct pf1 in rest frame of W-             
      p1sq = P1(5)**2
      p2sq = P2(5)**2
      den = 1/(2*xm1)
      h(4) = (sv1 + p1sq - p2sq)*den
      s1 = SQRT(1d0 - c1**2)
      hv = SQRT(h(4)**2 - p1sq)
      h(3) = hv*c1
      h(2) = hv*s1*COS(f1)
      h(1) = hv*s1*SIN(f1)
! Boost vector h to its LAB frame value pf1
      CALL BOOST(h,q1,pf1)
! Construct pf2 from momentum conservation
      DO k = 1,4
        pf2(k) = q1(k) - pf1(k)
      ENDDO
! Construct pf3 in rest frame of W+             
      p3sq = P3(5)**2
      p4sq = P4(5)**2
      den = 1/(2*xm2)
      h(4) = (sv2 + p3sq - p4sq)*den
      s2 = SQRT(1d0 - c2**2)
      hv = SQRT(h(4)**2 - p3sq)
      h(3) = hv*c2
      h(2) = hv*s2*COS(f2)
      h(1) = hv*s2*SIN(f2)
! Boost vector h to its LAB frame value pf3
      CALL BOOST(h,q2,pf3)
! Construct pf4 from momentum conservation
      DO k = 1,4
        pf4(k) = q2(k) - pf3(k)
      ENDDO
! Copy photons 4-momenta
      nphot = nsphot
      DO i = 1,nphot
        DO k = 1,4
          phot(i,k) = sphot(i,k)
        ENDDO
      ENDDO
      END

      SUBROUTINE DUMPD(NOUT)                 
C     **********************                 
C THIS PRINTS OUT FOUR MOMENTA OF PHOTONS    
C ON UNIT NO. NOUT                           
C     **********************                 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON / MOMDEC / pf1(4),pf2(4),pf3(4),pf4(4),phot(100,4),nphot    
      SAVE   / MOMDEC /
      DIMENSION SUM(4)                       
      WRITE(NOUT,*) '=====================DUMPD===================='
      WRITE(NOUT,3100) 'PF1',(  PF1(  K),K=1,4) 
      WRITE(NOUT,3100) 'PF2',(  PF2(  K),K=1,4)
      WRITE(NOUT,3100) 'PF3',(  PF3(  K),K=1,4) 
      WRITE(NOUT,3100) 'PF4',(  PF4(  K),K=1,4) 
      DO 100 I=1,NPHOT                       
  100 WRITE(NOUT,3100) 'PHO',(PHOT(I,K),K=1,4) 
      DO 200 K=1,4                           
  200 SUM(K)=PF1(K)+PF2(K)+PF3(K)+PF4(K)                   
      DO 210 I=1,NPHOT                       
      DO 210 K=1,4                           
  210 SUM(K)=SUM(K)+PHOT(I,K)               
      WRITE(NOUT,3100) 'SUM',(  SUM(  K),K=1,4) 
 3100 FORMAT(1X,A3,1X,5F18.12)               
      END                                    
