      PROGRAM  MAIN
*     *************
!----------------------------------------------------------------------!
! A simple demonstration program for YFSZZ - a M.C. event generator   !
! for e+e- --> ZZ --> 4-fermions with YFS exponentiation.            !
!----------------------------------------------------------------------! 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
! Histograms in blank common   
      COMMON / cglib / b(50000)
! Input/output files
      COMMON / INOUT  / NINP,NOUT
! Initialization of histogramming package --
! Here we use double precision HBOOK-like histogramming/plotting
! package GLIBK written by S. Jadach, (1990),  unpublished.
      CALL GLIMIT(50000) 
      NINP = 5
! This open on unix machine is not necessary
!#    OPEN(NINP)
! General Output for everybody including Glibk
      NOUT = 16
      OPEN(NOUT,FILE='./demo.output')
      REWIND(NOUT)
      CALL goutpu(NOUT)
      CALL demo
      END

      SUBROUTINE demo
*     ***************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON / INOUT  / NINP,NOUT
      COMMON / WGTALL / WTMOD,WTCRU1,WTCRU2,WTSET(100)
      SAVE   / INOUT  /,/ WGTALL /
      DIMENSION  xpar(100),npar(100)
!----------------------------------------------------------------------!
! Setting up the anomalous couplings as given in the paper:            !
!     K. Hagiwara, R.D. Peccei, D. Zeppenfeld and K. Hikasa,           !
!                 Nucl. Phys. B282 (1987) 253.                         !
! The variables used in this routine correspond to the following       !
! contants defined in the above paper:                                 !
!           constant name     corresponding variable                   !
!                g_4^V                g4(2)                            !
!                g_5^V                g5(2)                            !
! Note: f_4^V = g_4^V, f_5^V = g_5^V                                   !
!----------------------------------------------------------------------!
! Example of wild random anomalous coupling constants (benchmark)
! Must set KeyACC=1 in order to activate them.
!----------------------------------------------------------------------
      DIMENSION  ypar(100)
!   ZZgamma vertex Real part:   g4     g5  
      DATA (ypar(i),i=21,22)  / 0.01, 0.01 /
!   ZZgamma vertex Imag. part:  g4     g5   
      DATA (ypar(i),i=31,32)  / 0.00,  0.00 /
!   ZZZ vertex Real part:       g4     g5  
      DATA (ypar(i),i=41,42)  / 0.01,  0.01 /
!   WWZ vertex Imag. part:      g4     g5 
      DATA (ypar(i),i=51,52)  / 0.00,  0.00 /
!----------------------------------------------------------------------
! Nota Bene Standard Model looks as follows
!   ZZgamma vertex Real part:   g4    g5  
!     DATA (ypar(i),i=21,22)  / 0.0 , 0.0 /
!   ZZgamma vertex Imag. part:  g4    g5   
!     DATA (ypar(i),i=31,32)  / 0.0,  0.0 /
!   ZZZ vertex Real part:       g4    g5  
!     DATA (ypar(i),i=41,42)  / 0.0,  0.0 /
!   WWZ vertex Imag. part:      g4    g5 
!     DATA (ypar(i),i=51,52)  / 0.0,  0.0 /
!----------------------------------------------------------------------

!... Number of events to be generated
      NEVT = 100000
      WRITE(6,*)' ---------------- demo -------------------' 
      WRITE(6,*)' NEVT = ',NEVT,' events to be generated ...'
      WRITE(6,*)
      WRITE(NOUT,*)' ---------------- demo -------------------' 
      WRITE(NOUT,*)' NEVT = ',NEVT,' events to be generated ...'
      WRITE(NOUT,*)
!... Input parameters
      KeyCor = 1         ! =0/1 -- the main QED switch: Born/ISR
      KeyDZ1 = 4         ! =1/2/3/4/5/6/7/8/9/10/11 -- 1st Z decay channel
      KeyDZ2 = 5         ! =1/2/3/4/5/6/7/8/9/10/11 -- 2nd Z decay channel
                         !  1-v_e, 2-v_mu, 3-v_tau 4-e, 5-mu, 6-tau
                         !  7-u, 8-d, 9-c, 10-s, 11-b
      KeyAcc = 0         ! =0/1       -- Anomalous ZZV couplings OFF/ON
      KeyWgt = 1         ! =0/1       -- unweighted/weighted events
      CMSEne = 500.0     ! CMS energy
      amaZ   = 91.1888   ! Z mass
      GammZ  = 2.4974    ! Z width
      amaW   = 80.23     ! W mass
      GammW  = 2.03367033! W width
      sinW2  = 0.23103091! sin^2(theta_Weinberg)
      alfZin = 128.90    ! 1/alpha_Z
      vvmin  = 1D-6      ! min. radiative photon energy over beam energy  
      vvmax  = 0.99      ! max. radiative photon energy over beam energy
      WTMAX  = 15.0      ! max. weight for rejection
!
      npar( 1) = KeyCor
      npar( 2) = KeyDZ1
      npar( 3) = KeyDZ2
      npar( 4) = KeyAcc
      npar( 5) = KeyWgt
      xpar( 1) = CMSEne
      xpar( 2) = amaZ
      xpar( 3) = GammZ
      xpar( 4) = amaW
      xpar( 5) = GammW
      xpar( 6) = sinW2
      xpar( 7) = alfZin
      xpar( 8) = vvmin
      xpar( 9) = vvmax
      xpar(10) = WTMAX
!-------------------------------------------------------!
! Anomalous coupling constants
! They will be activated in the matrix element only if KeyAcc=1
      DO i=1,2
         xpar(20+i)=ypar(20+i)
         xpar(30+i)=ypar(30+i)
         xpar(40+i)=ypar(40+i)
         xpar(50+i)=ypar(50+i)
      ENDDO
!-------------------------------------------------------!

! ... Initialization
      CALL yfszz(-1,xpar,npar)
!     ************************
      CALL test1(-1,xpar,npar)
      IEV=0
      IED=0
! ... Event generation
      DO I = 1,NEVT
        IEV = IEV + 1
        IF(MOD(IEV,10000).EQ.1) WRITE(6 ,*) IEV
        CALL yfszz( 0,xpar,npar)
!       ************************
        CALL test1( 0,xpar,npar)
        IF (WTMOD.NE.0d0) THEN
          IED = IED + 1
          IF (IED.LE.5) CALL DUMPS(6)
          IF (IED.LE.5) CALL DUMPS(NOUT)
          IF (IED.LE.5) CALL DUMPD(6)
          IF (IED.LE.5) CALL DUMPD(NOUT)
        ENDIF
      ENDDO
! ... Final outputs
      CALL yfszz( 2,xpar,npar)
!     ************************
      CALL test1( 1,xpar,npar)
!
      XSEC = xpar(10)
      DSEC = XSEC*xpar(11)
      WRITE(6,'(A)') '........................................'
      WRITE(6,4000) '   XSEC = ',XSEC,' +- ',DSEC,' fb'
      WRITE(6,'(A)') '........................................'
 4000 FORMAT(A,F10.6,A4,F10.6,A3)
      END

      SUBROUTINE test1(mode,xpar,npar)
*     ********************************
!---------------------------------------------------------------!
! Some histograms ...                                           !
!---------------------------------------------------------------!
      IMPLICIT REAL*8 (A-H,O-Z) 
      PARAMETER( pi = 3.1415926535897932d0 )
! Input/output files
      COMMON / INOUT  / NINP,NOUT
      COMMON / MOMSET / q1(4),q2(4),SPHUM(4),phot(100,4),nphot
      COMMON / WGTALL / WTMOD,WTCRU1,WTCRU2,WTSET(100)  
      COMMON / VVREC / VVMIN,VVMAX,VV,BETI
      SAVE / INOUT /, / MOMSET /, / WGTALL /, / VVREC /
      DIMENSION  xpar(*),npar(*)
!
      IF (mode.EQ.-1) THEN
!---------------------------------------------------------------------- 
! Book histograms
        Nbin = 50
! Deg --> rad    
        degrad = pi/180
        th1 = 0
        th2 = pi
        CALL GBOOK1( 999,' WT $',Nbin,-2d0,18d0)
        CALL GBOOK1(1000,'1st Z: Theta distr. (rad) $',Nbin,th1,th2)
        CALL GBOOK1(1100,'2nd Z: Theta distr. (rad) $',Nbin,th1,th2)
        Ngmax = 10
        CALL GBOOK1(1200,'Photon multiplicity $',Ngmax,0d0,Ngmax*1d0)
      ELSEIF (mode.EQ.0) THEN
!---------------------------------------------------------------------- 
! Calculate angles for events with non-zero weight
        wtcrud = WTCRU1*WTCRU2
        IF (wtcrud.NE.0d0) THEN
          theZ1= DACOS(q1(3)/DSQRT(q1(1)**2+q1(2)**2+q1(3)**2)) 
          theZ2= DACOS(q2(3)/DSQRT(q2(1)**2+q2(2)**2+q2(3)**2)) 
        ELSE
          theZ1 = -1
          theZ2 = -1
        ENDIF
        wt = WTMOD
        CALL gf1( 999,wt,1d0)
! Fill histograms
! Theta distribution
        CALL gf1(1000,theZ1,wt)
        CALL gf1(1100,theZ2,wt)
! Photon multiplicity distribution
        CALL gf1(1200,nphot*1.1d0,wt)
      ELSEIF (mode.EQ.1) THEN 
        Xsec   = xpar(10) 
        Erel   = xpar(11)
        Nevtot = npar(10)
        Xcru   = xpar(20)
! Print histograms
        CALL gprint( 999)
        CALL gprint(1000)
        CALL gprint(1100)
        CALL gprint(1200)
! Print x-section
        WRITE(nout,*)'|||||||||||||||||||||||||||||||||||||||||||||||||'
        WRITE(nout,'(a,I15)')'   Nevent_accept = ',Nevtot
        WRITE(nout,'(a,f15.8,a)')'   Xsec_accep = ',Xsec     ,' fb.'
     $                          ,'   error      = ',Xsec*Erel,' fb.'
        WRITE(nout,*)'|||||||||||||||||||||||||||||||||||||||||||||||||'
!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
! The part below is optional i.e. it can be commented out.
! Up to four interesting plots will be  produced in LaTeX format
!----------------------------------------------------------------------
!----Initialize GPLOT               
        CALL gplint(0)                 
        noufig=11
        OPEN(noufig,file='demo.tex')
        CALL gplcap(-noufig)
! Re-Normalize properly histos in femtobarns
        xrange = th2 - th1          
        FacNor  = Nbin/(Nevtot*ABS(xrange)) *Xcru
        CALL Gopera(1000,'+',1000,2000,FacNor,0d0)
        FacNor  = Nbin/(Nevtot*ABS(xrange)) *Xcru
        CALL Gopera(1100,'+',1100,2100,FacNor,0d0)
        xrange = Ngmax
        Nbin   = Ngmax
        FacNor  = Nbin/(Nevtot*ABS(xrange)) *Xcru
        CALL Gopera(1200,'+',1200,2200,FacNor,0d0)
        xrange = 20d0
        Nbin   = Ngmax
        FacNor  = Nbin/(Nevtot*ABS(xrange)) 
        CALL Gopera( 999,'+', 999,1999,FacNor,0d0)
! Plot histos
        CALL GPLTIT('1st Z: d(sigma)/d(theta) [fb/rad] $')        
        CALL gminim(2000, 0.0d0)
        CALL gmaxim(2000,1.2d0)
        CALL gplset('DMOD',1d0)
        CALL gidopt(2000,'ERRO')
        CALL gplot( 2000,' ','B',0)
!
        CALL GPLTIT('2nd Z: d(sigma)/d(theta) [fb/rad] $')        
        CALL gminim(2100, 0.0d0)
        CALL gmaxim(2100,1.2d0)
        CALL gplset('DMOD',1d0)
        CALL gidopt(2100,'ERRO')
        CALL gplot( 2100,' ','B',0)
!
        CALL GPLTIT('d(sigma)/d(Ngam), Ngam - photon multipl. [fb]$')        
        CALL gminim(2200, 0.0d0)
        CALL gmaxim(2200, 0.5d0)
        CALL gplset('DMOD',2d0)
        CALL gidopt(2200,'ERRO')
        CALL gplot( 2200,' ','B',0)

        CALL GPLTIT(' Weight $')        
        CALL gminim(1999, 0.0d0)
        CALL gmaxim(1999, 0.5d0)
        CALL gplset('DMOD',2d0)
        CALL gidopt(1999,'ERRO')
        CALL gplot( 1999,' ','B',0)
!---- Close TeX file and output
        CALL gplend
        CLOSE(nout)
      ELSE
         WRITE(6,*)' >> test1: Wrong mode = ',mode
         STOP
      ENDIF
      END

